from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import (
    VariableOr,
    VariableOrList,
    VariableOrOptional,
)
from databricks.bundles.jobs._models.sql_task_subscription import (
    SqlTaskSubscription,
    SqlTaskSubscriptionParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class SqlTaskDashboard:
    """"""

    dashboard_id: VariableOr[str]
    """
    The canonical identifier of the SQL dashboard.
    """

    custom_subject: VariableOrOptional[str] = None
    """
    Subject of the email sent to subscribers of this task.
    """

    pause_subscriptions: VariableOrOptional[bool] = None
    """
    If true, the dashboard snapshot is not taken, and emails are not sent to subscribers.
    """

    subscriptions: VariableOrList[SqlTaskSubscription] = field(default_factory=list)
    """
    If specified, dashboard snapshots are sent to subscriptions.
    """

    @classmethod
    def from_dict(cls, value: "SqlTaskDashboardDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "SqlTaskDashboardDict":
        return _transform_to_json_value(self)  # type:ignore


class SqlTaskDashboardDict(TypedDict, total=False):
    """"""

    dashboard_id: VariableOr[str]
    """
    The canonical identifier of the SQL dashboard.
    """

    custom_subject: VariableOrOptional[str]
    """
    Subject of the email sent to subscribers of this task.
    """

    pause_subscriptions: VariableOrOptional[bool]
    """
    If true, the dashboard snapshot is not taken, and emails are not sent to subscribers.
    """

    subscriptions: VariableOrList[SqlTaskSubscriptionParam]
    """
    If specified, dashboard snapshots are sent to subscriptions.
    """


SqlTaskDashboardParam = SqlTaskDashboardDict | SqlTaskDashboard



from enum import Enum
from typing import Literal


class RunIf(Enum):
    """
    An optional value indicating the condition that determines whether the task should be run once its dependencies have been completed. When omitted, defaults to `ALL_SUCCESS`.

    Possible values are:
    * `ALL_SUCCESS`: All dependencies have executed and succeeded
    * `AT_LEAST_ONE_SUCCESS`: At least one dependency has succeeded
    * `NONE_FAILED`: None of the dependencies have failed and at least one was executed
    * `ALL_DONE`: All dependencies have been completed
    * `AT_LEAST_ONE_FAILED`: At least one dependency failed
    * `ALL_FAILED`: ALl dependencies have failed
    """

    ALL_SUCCESS = "ALL_SUCCESS"
    ALL_DONE = "ALL_DONE"
    NONE_FAILED = "NONE_FAILED"
    AT_LEAST_ONE_SUCCESS = "AT_LEAST_ONE_SUCCESS"
    ALL_FAILED = "ALL_FAILED"
    AT_LEAST_ONE_FAILED = "AT_LEAST_ONE_FAILED"


RunIfParam = (
    Literal[
        "ALL_SUCCESS",
        "ALL_DONE",
        "NONE_FAILED",
        "AT_LEAST_ONE_SUCCESS",
        "ALL_FAILED",
        "AT_LEAST_ONE_FAILED",
    ]
    | RunIf
)



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr, VariableOrOptional
from databricks.bundles.pipelines._models.pipeline_cluster_autoscale_mode import (
    PipelineClusterAutoscaleMode,
    PipelineClusterAutoscaleModeParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class PipelineClusterAutoscale:
    """"""

    max_workers: VariableOr[int]
    """
    The maximum number of workers to which the cluster can scale up when overloaded. `max_workers` must be strictly greater than `min_workers`.
    """

    min_workers: VariableOr[int]
    """
    The minimum number of workers the cluster can scale down to when underutilized.
    It is also the initial number of workers the cluster will have after creation.
    """

    mode: VariableOrOptional[PipelineClusterAutoscaleMode] = None
    """
    Databricks Enhanced Autoscaling optimizes cluster utilization by automatically
    allocating cluster resources based on workload volume, with minimal impact to
    the data processing latency of your pipelines. Enhanced Autoscaling is available
    for `updates` clusters only. The legacy autoscaling feature is used for `maintenance`
    clusters.
    """

    @classmethod
    def from_dict(cls, value: "PipelineClusterAutoscaleDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "PipelineClusterAutoscaleDict":
        return _transform_to_json_value(self)  # type:ignore


class PipelineClusterAutoscaleDict(TypedDict, total=False):
    """"""

    max_workers: VariableOr[int]
    """
    The maximum number of workers to which the cluster can scale up when overloaded. `max_workers` must be strictly greater than `min_workers`.
    """

    min_workers: VariableOr[int]
    """
    The minimum number of workers the cluster can scale down to when underutilized.
    It is also the initial number of workers the cluster will have after creation.
    """

    mode: VariableOrOptional[PipelineClusterAutoscaleModeParam]
    """
    Databricks Enhanced Autoscaling optimizes cluster utilization by automatically
    allocating cluster resources based on workload volume, with minimal impact to
    the data processing latency of your pipelines. Enhanced Autoscaling is available
    for `updates` clusters only. The legacy autoscaling feature is used for `maintenance`
    clusters.
    """


PipelineClusterAutoscaleParam = PipelineClusterAutoscaleDict | PipelineClusterAutoscale



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr, VariableOrOptional

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class IngestionGatewayPipelineDefinition:
    """
    :meta private: [EXPERIMENTAL]
    """

    connection_name: VariableOr[str]
    """
    Immutable. The Unity Catalog connection that this gateway pipeline uses to communicate with the source.
    """

    gateway_storage_catalog: VariableOr[str]
    """
    Required, Immutable. The name of the catalog for the gateway pipeline's storage location.
    """

    gateway_storage_schema: VariableOr[str]
    """
    Required, Immutable. The name of the schema for the gateway pipelines's storage location.
    """

    gateway_storage_name: VariableOrOptional[str] = None
    """
    Optional. The Unity Catalog-compatible name for the gateway storage location.
    This is the destination to use for the data that is extracted by the gateway.
    Delta Live Tables system will automatically create the storage location under the catalog and schema.
    """

    @classmethod
    def from_dict(cls, value: "IngestionGatewayPipelineDefinitionDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "IngestionGatewayPipelineDefinitionDict":
        return _transform_to_json_value(self)  # type:ignore


class IngestionGatewayPipelineDefinitionDict(TypedDict, total=False):
    """"""

    connection_name: VariableOr[str]
    """
    Immutable. The Unity Catalog connection that this gateway pipeline uses to communicate with the source.
    """

    gateway_storage_catalog: VariableOr[str]
    """
    Required, Immutable. The name of the catalog for the gateway pipeline's storage location.
    """

    gateway_storage_schema: VariableOr[str]
    """
    Required, Immutable. The name of the schema for the gateway pipelines's storage location.
    """

    gateway_storage_name: VariableOrOptional[str]
    """
    Optional. The Unity Catalog-compatible name for the gateway storage location.
    This is the destination to use for the data that is extracted by the gateway.
    Delta Live Tables system will automatically create the storage location under the catalog and schema.
    """


IngestionGatewayPipelineDefinitionParam = (
    IngestionGatewayPipelineDefinitionDict | IngestionGatewayPipelineDefinition
)



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr
from databricks.bundles.jobs._models.jobs_health_metric import (
    JobsHealthMetric,
    JobsHealthMetricParam,
)
from databricks.bundles.jobs._models.jobs_health_operator import (
    JobsHealthOperator,
    JobsHealthOperatorParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class JobsHealthRule:
    """"""

    metric: VariableOr[JobsHealthMetric]

    op: VariableOr[JobsHealthOperator]

    value: VariableOr[int]
    """
    Specifies the threshold value that the health metric should obey to satisfy the health rule.
    """

    @classmethod
    def from_dict(cls, value: "JobsHealthRuleDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "JobsHealthRuleDict":
        return _transform_to_json_value(self)  # type:ignore


class JobsHealthRuleDict(TypedDict, total=False):
    """"""

    metric: VariableOr[JobsHealthMetricParam]

    op: VariableOr[JobsHealthOperatorParam]

    value: VariableOr[int]
    """
    Specifies the threshold value that the health metric should obey to satisfy the health rule.
    """


JobsHealthRuleParam = JobsHealthRuleDict | JobsHealthRule



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr, VariableOrOptional

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class FileArrivalTriggerConfiguration:
    """"""

    url: VariableOr[str]
    """
    URL to be monitored for file arrivals. The path must point to the root or a subpath of the external location.
    """

    min_time_between_triggers_seconds: VariableOrOptional[int] = None
    """
    If set, the trigger starts a run only after the specified amount of time passed since
    the last time the trigger fired. The minimum allowed value is 60 seconds
    """

    wait_after_last_change_seconds: VariableOrOptional[int] = None
    """
    If set, the trigger starts a run only after no file activity has occurred for the specified amount of time.
    This makes it possible to wait for a batch of incoming files to arrive before triggering a run. The
    minimum allowed value is 60 seconds.
    """

    @classmethod
    def from_dict(cls, value: "FileArrivalTriggerConfigurationDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "FileArrivalTriggerConfigurationDict":
        return _transform_to_json_value(self)  # type:ignore


class FileArrivalTriggerConfigurationDict(TypedDict, total=False):
    """"""

    url: VariableOr[str]
    """
    URL to be monitored for file arrivals. The path must point to the root or a subpath of the external location.
    """

    min_time_between_triggers_seconds: VariableOrOptional[int]
    """
    If set, the trigger starts a run only after the specified amount of time passed since
    the last time the trigger fired. The minimum allowed value is 60 seconds
    """

    wait_after_last_change_seconds: VariableOrOptional[int]
    """
    If set, the trigger starts a run only after no file activity has occurred for the specified amount of time.
    This makes it possible to wait for a batch of incoming files to arrive before triggering a run. The
    minimum allowed value is 60 seconds.
    """


FileArrivalTriggerConfigurationParam = (
    FileArrivalTriggerConfigurationDict | FileArrivalTriggerConfiguration
)



import typing
from dataclasses import dataclass, fields, is_dataclass
from typing import (
    Generic,
    Optional,
    Type,
    TypeVar,
    Union,
)

__all__ = [
    "Variable",
    "VariableOr",
    "VariableOrOptional",
    "VariableOrDict",
    "VariableOrList",
    "variables",
]

_T = TypeVar("_T", covariant=True)


@dataclass(kw_only=True)
class Variable(Generic[_T]):
    """
    Reference to a bundle variable.

    See: `Databricks Asset Bundles configuration <https://docs.databricks.com/en/dev-tools/bundles/settings.html>`_
    """

    path: str
    """
    Path to the variable, e.g. "var.my_variable".
    """

    type: Type[_T]
    """
    Type of the variable.
    """

    def __str__(self):
        return self.value

    def __repr__(self):
        return self.value

    @property
    def value(self) -> str:
        """
        Returns the variable path in the format of "${path}"
        """

        return "${" + self.path + "}"


VariableOr = Union[Variable[_T], _T]
VariableOrOptional = Union[Variable[_T], Optional[_T]]

# - 1. variable: ${var.my_list}
# - 2. regular list: [{"name": "abc"}, ...]
# - 3. list of variables: ["${var.my_item}", ...]
# - 4. list with a mix of (3) and (4) as elements
VariableOrList = VariableOr[list[VariableOr[_T]]]

# - 1. variable: ${var.my_list}
# - 2. regular dict: {"key": "value", ...}
# - 3. dict with variables (but not as keys): {"value": "${var.my_item}", ...}
# - 4. dict with a mix of (3) and (4) as values
VariableOrDict = VariableOr[dict[str, VariableOr[_T]]]


def variables(cls: type[_T]) -> type[_T]:
    """
    A decorator that initializes each annotated attribute in a class
    with :class:`~Variable` type. Variables are initialized with a path
    that corresponds to the attribute name. Variables should specify their
    type, or else they will be treated as :class:`~Any`. Complex types
    like data classes, lists or dictionaries are supported.

    For example, if your databricks.yml file contains:

    .. code-block:: yaml

        variables:
          warehouse_id:
            description: Warehouse ID for SQL tasks
            default: ...

    You can define a class with a `warehouse_id` attribute:

    .. code-block:: python

        @variables
        class MyVariables:
          warehouse_id: Variable[str] # ${var.warehouse_id}

    And later use it in your code as `MyVariables.warehouse_id`.

    For accessing bundle variable values, see :meth:`Bundle.resolve_variable`.
    """

    # making class a dataclass, solves a lot of problems, because we can just use 'fields'
    cls = dataclass(cls)
    assert is_dataclass(cls)

    # don't get type hints unless needed
    hints = None

    for field in fields(cls):
        field_type = field.type

        if isinstance(field_type, typing.ForwardRef) or isinstance(field_type, str):
            if hints is None:
                hints = typing.get_type_hints(cls)

            field_type = hints.get(field.name, field.type)

        origin = typing.get_origin(field_type) or field_type

        if origin != Variable:
            raise ValueError(
                f"Only 'Variable' type is allowed in classes annotated with @variables, got {field_type}"
            )

        args = typing.get_args(field_type)

        if not args:
            raise ValueError(
                f"Variable type must be specified for '{field.name}', e.g. Variable[str]"
            )
        else:
            variable_type = args[0]

        variable = Variable(path=f"var.{field.name}", type=variable_type)

        setattr(cls, field.name, variable)

    return cls



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.compute._models.cluster_spec import (
    ClusterSpec,
    ClusterSpecParam,
)
from databricks.bundles.compute._models.library import (
    Library,
    LibraryParam,
)
from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import (
    VariableOr,
    VariableOrList,
    VariableOrOptional,
)
from databricks.bundles.jobs._models.clean_rooms_notebook_task import (
    CleanRoomsNotebookTask,
    CleanRoomsNotebookTaskParam,
)
from databricks.bundles.jobs._models.condition_task import (
    ConditionTask,
    ConditionTaskParam,
)
from databricks.bundles.jobs._models.dashboard_task import (
    DashboardTask,
    DashboardTaskParam,
)
from databricks.bundles.jobs._models.dbt_cloud_task import (
    DbtCloudTask,
    DbtCloudTaskParam,
)
from databricks.bundles.jobs._models.dbt_task import DbtTask, DbtTaskParam
from databricks.bundles.jobs._models.for_each_task import (
    ForEachTask,
    ForEachTaskParam,
)
from databricks.bundles.jobs._models.gen_ai_compute_task import (
    GenAiComputeTask,
    GenAiComputeTaskParam,
)
from databricks.bundles.jobs._models.jobs_health_rules import (
    JobsHealthRules,
    JobsHealthRulesParam,
)
from databricks.bundles.jobs._models.notebook_task import (
    NotebookTask,
    NotebookTaskParam,
)
from databricks.bundles.jobs._models.pipeline_task import (
    PipelineTask,
    PipelineTaskParam,
)
from databricks.bundles.jobs._models.power_bi_task import PowerBiTask, PowerBiTaskParam
from databricks.bundles.jobs._models.python_wheel_task import (
    PythonWheelTask,
    PythonWheelTaskParam,
)
from databricks.bundles.jobs._models.run_if import RunIf, RunIfParam
from databricks.bundles.jobs._models.run_job_task import (
    RunJobTask,
    RunJobTaskParam,
)
from databricks.bundles.jobs._models.spark_jar_task import (
    SparkJarTask,
    SparkJarTaskParam,
)
from databricks.bundles.jobs._models.spark_python_task import (
    SparkPythonTask,
    SparkPythonTaskParam,
)
from databricks.bundles.jobs._models.spark_submit_task import (
    SparkSubmitTask,
    SparkSubmitTaskParam,
)
from databricks.bundles.jobs._models.sql_task import SqlTask, SqlTaskParam
from databricks.bundles.jobs._models.task_dependency import (
    TaskDependency,
    TaskDependencyParam,
)
from databricks.bundles.jobs._models.task_email_notifications import (
    TaskEmailNotifications,
    TaskEmailNotificationsParam,
)
from databricks.bundles.jobs._models.task_notification_settings import (
    TaskNotificationSettings,
    TaskNotificationSettingsParam,
)
from databricks.bundles.jobs._models.webhook_notifications import (
    WebhookNotifications,
    WebhookNotificationsParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class Task:
    """"""

    task_key: VariableOr[str]
    """
    A unique name for the task. This field is used to refer to this task from other tasks.
    This field is required and must be unique within its parent job.
    On Update or Reset, this field is used to reference the tasks to be updated or reset.
    """

    clean_rooms_notebook_task: VariableOrOptional[CleanRoomsNotebookTask] = None
    """
    The task runs a [clean rooms](https://docs.databricks.com/en/clean-rooms/index.html) notebook
    when the `clean_rooms_notebook_task` field is present.
    """

    condition_task: VariableOrOptional[ConditionTask] = None
    """
    The task evaluates a condition that can be used to control the execution of other tasks when the `condition_task` field is present.
    The condition task does not require a cluster to execute and does not support retries or notifications.
    """

    dashboard_task: VariableOrOptional[DashboardTask] = None
    """
    The task refreshes a dashboard and sends a snapshot to subscribers.
    """

    dbt_cloud_task: VariableOrOptional[DbtCloudTask] = None
    """
    :meta private: [EXPERIMENTAL]
    
    Task type for dbt cloud
    """

    dbt_task: VariableOrOptional[DbtTask] = None
    """
    The task runs one or more dbt commands when the `dbt_task` field is present. The dbt task requires both Databricks SQL and the ability to use a serverless or a pro SQL warehouse.
    """

    depends_on: VariableOrList[TaskDependency] = field(default_factory=list)
    """
    An optional array of objects specifying the dependency graph of the task. All tasks specified in this field must complete before executing this task. The task will run only if the `run_if` condition is true.
    The key is `task_key`, and the value is the name assigned to the dependent task.
    """

    description: VariableOrOptional[str] = None
    """
    An optional description for this task.
    """

    disable_auto_optimization: VariableOrOptional[bool] = None
    """
    An option to disable auto optimization in serverless
    """

    email_notifications: VariableOrOptional[TaskEmailNotifications] = None
    """
    An optional set of email addresses that is notified when runs of this task begin or complete as well as when this task is deleted. The default behavior is to not send any emails.
    """

    environment_key: VariableOrOptional[str] = None
    """
    The key that references an environment spec in a job. This field is required for Python script, Python wheel and dbt tasks when using serverless compute.
    """

    existing_cluster_id: VariableOrOptional[str] = None
    """
    If existing_cluster_id, the ID of an existing cluster that is used for all runs.
    When running jobs or tasks on an existing cluster, you may need to manually restart
    the cluster if it stops responding. We suggest running jobs and tasks on new clusters for
    greater reliability
    """

    for_each_task: VariableOrOptional[ForEachTask] = None
    """
    The task executes a nested task for every input provided when the `for_each_task` field is present.
    """

    gen_ai_compute_task: VariableOrOptional[GenAiComputeTask] = None
    """
    :meta private: [EXPERIMENTAL]
    """

    health: VariableOrOptional[JobsHealthRules] = None

    job_cluster_key: VariableOrOptional[str] = None
    """
    If job_cluster_key, this task is executed reusing the cluster specified in `job.settings.job_clusters`.
    """

    libraries: VariableOrList[Library] = field(default_factory=list)
    """
    An optional list of libraries to be installed on the cluster.
    The default value is an empty list.
    """

    max_retries: VariableOrOptional[int] = None
    """
    An optional maximum number of times to retry an unsuccessful run. A run is considered to be unsuccessful if it completes with the `FAILED` result_state or `INTERNAL_ERROR` `life_cycle_state`. The value `-1` means to retry indefinitely and the value `0` means to never retry.
    """

    min_retry_interval_millis: VariableOrOptional[int] = None
    """
    An optional minimal interval in milliseconds between the start of the failed run and the subsequent retry run. The default behavior is that unsuccessful runs are immediately retried.
    """

    new_cluster: VariableOrOptional[ClusterSpec] = None
    """
    If new_cluster, a description of a new cluster that is created for each run.
    """

    notebook_task: VariableOrOptional[NotebookTask] = None
    """
    The task runs a notebook when the `notebook_task` field is present.
    """

    notification_settings: VariableOrOptional[TaskNotificationSettings] = None
    """
    Optional notification settings that are used when sending notifications to each of the `email_notifications` and `webhook_notifications` for this task.
    """

    pipeline_task: VariableOrOptional[PipelineTask] = None
    """
    The task triggers a pipeline update when the `pipeline_task` field is present. Only pipelines configured to use triggered more are supported.
    """

    power_bi_task: VariableOrOptional[PowerBiTask] = None
    """
    The task triggers a Power BI semantic model update when the `power_bi_task` field is present.
    """

    python_wheel_task: VariableOrOptional[PythonWheelTask] = None
    """
    The task runs a Python wheel when the `python_wheel_task` field is present.
    """

    retry_on_timeout: VariableOrOptional[bool] = None
    """
    An optional policy to specify whether to retry a job when it times out. The default behavior
    is to not retry on timeout.
    """

    run_if: VariableOrOptional[RunIf] = None
    """
    An optional value specifying the condition determining whether the task is run once its dependencies have been completed.
    
    * `ALL_SUCCESS`: All dependencies have executed and succeeded
    * `AT_LEAST_ONE_SUCCESS`: At least one dependency has succeeded
    * `NONE_FAILED`: None of the dependencies have failed and at least one was executed
    * `ALL_DONE`: All dependencies have been completed
    * `AT_LEAST_ONE_FAILED`: At least one dependency failed
    * `ALL_FAILED`: ALl dependencies have failed
    """

    run_job_task: VariableOrOptional[RunJobTask] = None
    """
    The task triggers another job when the `run_job_task` field is present.
    """

    spark_jar_task: VariableOrOptional[SparkJarTask] = None
    """
    The task runs a JAR when the `spark_jar_task` field is present.
    """

    spark_python_task: VariableOrOptional[SparkPythonTask] = None
    """
    The task runs a Python file when the `spark_python_task` field is present.
    """

    spark_submit_task: VariableOrOptional[SparkSubmitTask] = None
    """
    (Legacy) The task runs the spark-submit script when the `spark_submit_task` field is present. This task can run only on new clusters and is not compatible with serverless compute.
    
    In the `new_cluster` specification, `libraries` and `spark_conf` are not supported. Instead, use `--jars` and `--py-files` to add Java and Python libraries and `--conf` to set the Spark configurations.
    
    `master`, `deploy-mode`, and `executor-cores` are automatically configured by Databricks; you _cannot_ specify them in parameters.
    
    By default, the Spark submit job uses all available memory (excluding reserved memory for Databricks services). You can set `--driver-memory`, and `--executor-memory` to a smaller value to leave some room for off-heap usage.
    
    The `--jars`, `--py-files`, `--files` arguments support DBFS and S3 paths.
    """

    sql_task: VariableOrOptional[SqlTask] = None
    """
    The task runs a SQL query or file, or it refreshes a SQL alert or a legacy SQL dashboard when the `sql_task` field is present.
    """

    timeout_seconds: VariableOrOptional[int] = None
    """
    An optional timeout applied to each run of this job task. A value of `0` means no timeout.
    """

    webhook_notifications: VariableOrOptional[WebhookNotifications] = None
    """
    A collection of system notification IDs to notify when runs of this task begin or complete. The default behavior is to not send any system notifications.
    """

    @classmethod
    def from_dict(cls, value: "TaskDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "TaskDict":
        return _transform_to_json_value(self)  # type:ignore


class TaskDict(TypedDict, total=False):
    """"""

    task_key: VariableOr[str]
    """
    A unique name for the task. This field is used to refer to this task from other tasks.
    This field is required and must be unique within its parent job.
    On Update or Reset, this field is used to reference the tasks to be updated or reset.
    """

    clean_rooms_notebook_task: VariableOrOptional[CleanRoomsNotebookTaskParam]
    """
    The task runs a [clean rooms](https://docs.databricks.com/en/clean-rooms/index.html) notebook
    when the `clean_rooms_notebook_task` field is present.
    """

    condition_task: VariableOrOptional[ConditionTaskParam]
    """
    The task evaluates a condition that can be used to control the execution of other tasks when the `condition_task` field is present.
    The condition task does not require a cluster to execute and does not support retries or notifications.
    """

    dashboard_task: VariableOrOptional[DashboardTaskParam]
    """
    The task refreshes a dashboard and sends a snapshot to subscribers.
    """

    dbt_cloud_task: VariableOrOptional[DbtCloudTaskParam]
    """
    :meta private: [EXPERIMENTAL]
    
    Task type for dbt cloud
    """

    dbt_task: VariableOrOptional[DbtTaskParam]
    """
    The task runs one or more dbt commands when the `dbt_task` field is present. The dbt task requires both Databricks SQL and the ability to use a serverless or a pro SQL warehouse.
    """

    depends_on: VariableOrList[TaskDependencyParam]
    """
    An optional array of objects specifying the dependency graph of the task. All tasks specified in this field must complete before executing this task. The task will run only if the `run_if` condition is true.
    The key is `task_key`, and the value is the name assigned to the dependent task.
    """

    description: VariableOrOptional[str]
    """
    An optional description for this task.
    """

    disable_auto_optimization: VariableOrOptional[bool]
    """
    An option to disable auto optimization in serverless
    """

    email_notifications: VariableOrOptional[TaskEmailNotificationsParam]
    """
    An optional set of email addresses that is notified when runs of this task begin or complete as well as when this task is deleted. The default behavior is to not send any emails.
    """

    environment_key: VariableOrOptional[str]
    """
    The key that references an environment spec in a job. This field is required for Python script, Python wheel and dbt tasks when using serverless compute.
    """

    existing_cluster_id: VariableOrOptional[str]
    """
    If existing_cluster_id, the ID of an existing cluster that is used for all runs.
    When running jobs or tasks on an existing cluster, you may need to manually restart
    the cluster if it stops responding. We suggest running jobs and tasks on new clusters for
    greater reliability
    """

    for_each_task: VariableOrOptional[ForEachTaskParam]
    """
    The task executes a nested task for every input provided when the `for_each_task` field is present.
    """

    gen_ai_compute_task: VariableOrOptional[GenAiComputeTaskParam]
    """
    :meta private: [EXPERIMENTAL]
    """

    health: VariableOrOptional[JobsHealthRulesParam]

    job_cluster_key: VariableOrOptional[str]
    """
    If job_cluster_key, this task is executed reusing the cluster specified in `job.settings.job_clusters`.
    """

    libraries: VariableOrList[LibraryParam]
    """
    An optional list of libraries to be installed on the cluster.
    The default value is an empty list.
    """

    max_retries: VariableOrOptional[int]
    """
    An optional maximum number of times to retry an unsuccessful run. A run is considered to be unsuccessful if it completes with the `FAILED` result_state or `INTERNAL_ERROR` `life_cycle_state`. The value `-1` means to retry indefinitely and the value `0` means to never retry.
    """

    min_retry_interval_millis: VariableOrOptional[int]
    """
    An optional minimal interval in milliseconds between the start of the failed run and the subsequent retry run. The default behavior is that unsuccessful runs are immediately retried.
    """

    new_cluster: VariableOrOptional[ClusterSpecParam]
    """
    If new_cluster, a description of a new cluster that is created for each run.
    """

    notebook_task: VariableOrOptional[NotebookTaskParam]
    """
    The task runs a notebook when the `notebook_task` field is present.
    """

    notification_settings: VariableOrOptional[TaskNotificationSettingsParam]
    """
    Optional notification settings that are used when sending notifications to each of the `email_notifications` and `webhook_notifications` for this task.
    """

    pipeline_task: VariableOrOptional[PipelineTaskParam]
    """
    The task triggers a pipeline update when the `pipeline_task` field is present. Only pipelines configured to use triggered more are supported.
    """

    power_bi_task: VariableOrOptional[PowerBiTaskParam]
    """
    The task triggers a Power BI semantic model update when the `power_bi_task` field is present.
    """

    python_wheel_task: VariableOrOptional[PythonWheelTaskParam]
    """
    The task runs a Python wheel when the `python_wheel_task` field is present.
    """

    retry_on_timeout: VariableOrOptional[bool]
    """
    An optional policy to specify whether to retry a job when it times out. The default behavior
    is to not retry on timeout.
    """

    run_if: VariableOrOptional[RunIfParam]
    """
    An optional value specifying the condition determining whether the task is run once its dependencies have been completed.
    
    * `ALL_SUCCESS`: All dependencies have executed and succeeded
    * `AT_LEAST_ONE_SUCCESS`: At least one dependency has succeeded
    * `NONE_FAILED`: None of the dependencies have failed and at least one was executed
    * `ALL_DONE`: All dependencies have been completed
    * `AT_LEAST_ONE_FAILED`: At least one dependency failed
    * `ALL_FAILED`: ALl dependencies have failed
    """

    run_job_task: VariableOrOptional[RunJobTaskParam]
    """
    The task triggers another job when the `run_job_task` field is present.
    """

    spark_jar_task: VariableOrOptional[SparkJarTaskParam]
    """
    The task runs a JAR when the `spark_jar_task` field is present.
    """

    spark_python_task: VariableOrOptional[SparkPythonTaskParam]
    """
    The task runs a Python file when the `spark_python_task` field is present.
    """

    spark_submit_task: VariableOrOptional[SparkSubmitTaskParam]
    """
    (Legacy) The task runs the spark-submit script when the `spark_submit_task` field is present. This task can run only on new clusters and is not compatible with serverless compute.
    
    In the `new_cluster` specification, `libraries` and `spark_conf` are not supported. Instead, use `--jars` and `--py-files` to add Java and Python libraries and `--conf` to set the Spark configurations.
    
    `master`, `deploy-mode`, and `executor-cores` are automatically configured by Databricks; you _cannot_ specify them in parameters.
    
    By default, the Spark submit job uses all available memory (excluding reserved memory for Databricks services). You can set `--driver-memory`, and `--executor-memory` to a smaller value to leave some room for off-heap usage.
    
    The `--jars`, `--py-files`, `--files` arguments support DBFS and S3 paths.
    """

    sql_task: VariableOrOptional[SqlTaskParam]
    """
    The task runs a SQL query or file, or it refreshes a SQL alert or a legacy SQL dashboard when the `sql_task` field is present.
    """

    timeout_seconds: VariableOrOptional[int]
    """
    An optional timeout applied to each run of this job task. A value of `0` means no timeout.
    """

    webhook_notifications: VariableOrOptional[WebhookNotificationsParam]
    """
    A collection of system notification IDs to notify when runs of this task begin or complete. The default behavior is to not send any system notifications.
    """


TaskParam = TaskDict | Task



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.compute._models.docker_basic_auth import (
    DockerBasicAuth,
    DockerBasicAuthParam,
)
from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class DockerImage:
    """"""

    basic_auth: VariableOrOptional[DockerBasicAuth] = None

    url: VariableOrOptional[str] = None
    """
    URL of the docker image.
    """

    @classmethod
    def from_dict(cls, value: "DockerImageDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "DockerImageDict":
        return _transform_to_json_value(self)  # type:ignore


class DockerImageDict(TypedDict, total=False):
    """"""

    basic_auth: VariableOrOptional[DockerBasicAuthParam]

    url: VariableOrOptional[str]
    """
    URL of the docker image.
    """


DockerImageParam = DockerImageDict | DockerImage



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrList, VariableOrOptional
from databricks.bundles.jobs._models.power_bi_model import (
    PowerBiModel,
    PowerBiModelParam,
)
from databricks.bundles.jobs._models.power_bi_table import (
    PowerBiTable,
    PowerBiTableParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class PowerBiTask:
    """"""

    connection_resource_name: VariableOrOptional[str] = None
    """
    The resource name of the UC connection to authenticate from Databricks to Power BI
    """

    power_bi_model: VariableOrOptional[PowerBiModel] = None
    """
    The semantic model to update
    """

    refresh_after_update: VariableOrOptional[bool] = None
    """
    Whether the model should be refreshed after the update
    """

    tables: VariableOrList[PowerBiTable] = field(default_factory=list)
    """
    The tables to be exported to Power BI
    """

    warehouse_id: VariableOrOptional[str] = None
    """
    The SQL warehouse ID to use as the Power BI data source
    """

    @classmethod
    def from_dict(cls, value: "PowerBiTaskDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "PowerBiTaskDict":
        return _transform_to_json_value(self)  # type:ignore


class PowerBiTaskDict(TypedDict, total=False):
    """"""

    connection_resource_name: VariableOrOptional[str]
    """
    The resource name of the UC connection to authenticate from Databricks to Power BI
    """

    power_bi_model: VariableOrOptional[PowerBiModelParam]
    """
    The semantic model to update
    """

    refresh_after_update: VariableOrOptional[bool]
    """
    Whether the model should be refreshed after the update
    """

    tables: VariableOrList[PowerBiTableParam]
    """
    The tables to be exported to Power BI
    """

    warehouse_id: VariableOrOptional[str]
    """
    The SQL warehouse ID to use as the Power BI data source
    """


PowerBiTaskParam = PowerBiTaskDict | PowerBiTask



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.compute._models.aws_availability import (
    AwsAvailability,
    AwsAvailabilityParam,
)
from databricks.bundles.compute._models.ebs_volume_type import (
    EbsVolumeType,
    EbsVolumeTypeParam,
)
from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class AwsAttributes:
    """
    Attributes set during cluster creation which are related to Amazon Web Services.
    """

    availability: VariableOrOptional[AwsAvailability] = None

    ebs_volume_count: VariableOrOptional[int] = None
    """
    The number of volumes launched for each instance. Users can choose up to 10 volumes.
    This feature is only enabled for supported node types. Legacy node types cannot specify
    custom EBS volumes.
    For node types with no instance store, at least one EBS volume needs to be specified;
    otherwise, cluster creation will fail.
    
    These EBS volumes will be mounted at `/ebs0`, `/ebs1`, and etc.
    Instance store volumes will be mounted at `/local_disk0`, `/local_disk1`, and etc.
    
    If EBS volumes are attached, Databricks will configure Spark to use only the EBS volumes for
    scratch storage because heterogenously sized scratch devices can lead to inefficient disk
    utilization. If no EBS volumes are attached, Databricks will configure Spark to use instance
    store volumes.
    
    Please note that if EBS volumes are specified, then the Spark configuration `spark.local.dir`
    will be overridden.
    """

    ebs_volume_iops: VariableOrOptional[int] = None
    """
    If using gp3 volumes, what IOPS to use for the disk. If this is not set, the maximum performance of a gp2 volume with the same volume size will be used.
    """

    ebs_volume_size: VariableOrOptional[int] = None
    """
    The size of each EBS volume (in GiB) launched for each instance. For general purpose
    SSD, this value must be within the range 100 - 4096. For throughput optimized HDD,
    this value must be within the range 500 - 4096.
    """

    ebs_volume_throughput: VariableOrOptional[int] = None
    """
    If using gp3 volumes, what throughput to use for the disk. If this is not set, the maximum performance of a gp2 volume with the same volume size will be used.
    """

    ebs_volume_type: VariableOrOptional[EbsVolumeType] = None

    first_on_demand: VariableOrOptional[int] = None
    """
    The first `first_on_demand` nodes of the cluster will be placed on on-demand instances.
    If this value is greater than 0, the cluster driver node in particular will be placed on an
    on-demand instance. If this value is greater than or equal to the current cluster size, all
    nodes will be placed on on-demand instances. If this value is less than the current cluster
    size, `first_on_demand` nodes will be placed on on-demand instances and the remainder will
    be placed on `availability` instances. Note that this value does not affect
    cluster size and cannot currently be mutated over the lifetime of a cluster.
    """

    instance_profile_arn: VariableOrOptional[str] = None
    """
    Nodes for this cluster will only be placed on AWS instances with this instance profile. If
    ommitted, nodes will be placed on instances without an IAM instance profile. The instance
    profile must have previously been added to the Databricks environment by an account
    administrator.
    
    This feature may only be available to certain customer plans.
    """

    spot_bid_price_percent: VariableOrOptional[int] = None
    """
    The bid price for AWS spot instances, as a percentage of the corresponding instance type's
    on-demand price.
    For example, if this field is set to 50, and the cluster needs a new `r3.xlarge` spot
    instance, then the bid price is half of the price of
    on-demand `r3.xlarge` instances. Similarly, if this field is set to 200, the bid price is twice
    the price of on-demand `r3.xlarge` instances. If not specified, the default value is 100.
    When spot instances are requested for this cluster, only spot instances whose bid price
    percentage matches this field will be considered.
    Note that, for safety, we enforce this field to be no more than 10000.
    """

    zone_id: VariableOrOptional[str] = None
    """
    Identifier for the availability zone/datacenter in which the cluster resides.
    This string will be of a form like "us-west-2a". The provided availability
    zone must be in the same region as the Databricks deployment. For example, "us-west-2a"
    is not a valid zone id if the Databricks deployment resides in the "us-east-1" region.
    This is an optional field at cluster creation, and if not specified, a default zone will be used.
    If the zone specified is "auto", will try to place cluster in a zone with high availability,
    and will retry placement in a different AZ if there is not enough capacity.
    
    The list of available zones as well as the default value can be found by using the
    `List Zones` method.
    """

    @classmethod
    def from_dict(cls, value: "AwsAttributesDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "AwsAttributesDict":
        return _transform_to_json_value(self)  # type:ignore


class AwsAttributesDict(TypedDict, total=False):
    """"""

    availability: VariableOrOptional[AwsAvailabilityParam]

    ebs_volume_count: VariableOrOptional[int]
    """
    The number of volumes launched for each instance. Users can choose up to 10 volumes.
    This feature is only enabled for supported node types. Legacy node types cannot specify
    custom EBS volumes.
    For node types with no instance store, at least one EBS volume needs to be specified;
    otherwise, cluster creation will fail.
    
    These EBS volumes will be mounted at `/ebs0`, `/ebs1`, and etc.
    Instance store volumes will be mounted at `/local_disk0`, `/local_disk1`, and etc.
    
    If EBS volumes are attached, Databricks will configure Spark to use only the EBS volumes for
    scratch storage because heterogenously sized scratch devices can lead to inefficient disk
    utilization. If no EBS volumes are attached, Databricks will configure Spark to use instance
    store volumes.
    
    Please note that if EBS volumes are specified, then the Spark configuration `spark.local.dir`
    will be overridden.
    """

    ebs_volume_iops: VariableOrOptional[int]
    """
    If using gp3 volumes, what IOPS to use for the disk. If this is not set, the maximum performance of a gp2 volume with the same volume size will be used.
    """

    ebs_volume_size: VariableOrOptional[int]
    """
    The size of each EBS volume (in GiB) launched for each instance. For general purpose
    SSD, this value must be within the range 100 - 4096. For throughput optimized HDD,
    this value must be within the range 500 - 4096.
    """

    ebs_volume_throughput: VariableOrOptional[int]
    """
    If using gp3 volumes, what throughput to use for the disk. If this is not set, the maximum performance of a gp2 volume with the same volume size will be used.
    """

    ebs_volume_type: VariableOrOptional[EbsVolumeTypeParam]

    first_on_demand: VariableOrOptional[int]
    """
    The first `first_on_demand` nodes of the cluster will be placed on on-demand instances.
    If this value is greater than 0, the cluster driver node in particular will be placed on an
    on-demand instance. If this value is greater than or equal to the current cluster size, all
    nodes will be placed on on-demand instances. If this value is less than the current cluster
    size, `first_on_demand` nodes will be placed on on-demand instances and the remainder will
    be placed on `availability` instances. Note that this value does not affect
    cluster size and cannot currently be mutated over the lifetime of a cluster.
    """

    instance_profile_arn: VariableOrOptional[str]
    """
    Nodes for this cluster will only be placed on AWS instances with this instance profile. If
    ommitted, nodes will be placed on instances without an IAM instance profile. The instance
    profile must have previously been added to the Databricks environment by an account
    administrator.
    
    This feature may only be available to certain customer plans.
    """

    spot_bid_price_percent: VariableOrOptional[int]
    """
    The bid price for AWS spot instances, as a percentage of the corresponding instance type's
    on-demand price.
    For example, if this field is set to 50, and the cluster needs a new `r3.xlarge` spot
    instance, then the bid price is half of the price of
    on-demand `r3.xlarge` instances. Similarly, if this field is set to 200, the bid price is twice
    the price of on-demand `r3.xlarge` instances. If not specified, the default value is 100.
    When spot instances are requested for this cluster, only spot instances whose bid price
    percentage matches this field will be considered.
    Note that, for safety, we enforce this field to be no more than 10000.
    """

    zone_id: VariableOrOptional[str]
    """
    Identifier for the availability zone/datacenter in which the cluster resides.
    This string will be of a form like "us-west-2a". The provided availability
    zone must be in the same region as the Databricks deployment. For example, "us-west-2a"
    is not a valid zone id if the Databricks deployment resides in the "us-east-1" region.
    This is an optional field at cluster creation, and if not specified, a default zone will be used.
    If the zone specified is "auto", will try to place cluster in a zone with high availability,
    and will retry placement in a different AZ if there is not enough capacity.
    
    The list of available zones as well as the default value can be found by using the
    `List Zones` method.
    """


AwsAttributesParam = AwsAttributesDict | AwsAttributes



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional
from databricks.bundles.jobs._models.authentication_method import (
    AuthenticationMethod,
    AuthenticationMethodParam,
)
from databricks.bundles.jobs._models.storage_mode import StorageMode, StorageModeParam

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class PowerBiModel:
    """"""

    authentication_method: VariableOrOptional[AuthenticationMethod] = None
    """
    How the published Power BI model authenticates to Databricks
    """

    model_name: VariableOrOptional[str] = None
    """
    The name of the Power BI model
    """

    overwrite_existing: VariableOrOptional[bool] = None
    """
    Whether to overwrite existing Power BI models
    """

    storage_mode: VariableOrOptional[StorageMode] = None
    """
    The default storage mode of the Power BI model
    """

    workspace_name: VariableOrOptional[str] = None
    """
    The name of the Power BI workspace of the model
    """

    @classmethod
    def from_dict(cls, value: "PowerBiModelDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "PowerBiModelDict":
        return _transform_to_json_value(self)  # type:ignore


class PowerBiModelDict(TypedDict, total=False):
    """"""

    authentication_method: VariableOrOptional[AuthenticationMethodParam]
    """
    How the published Power BI model authenticates to Databricks
    """

    model_name: VariableOrOptional[str]
    """
    The name of the Power BI model
    """

    overwrite_existing: VariableOrOptional[bool]
    """
    Whether to overwrite existing Power BI models
    """

    storage_mode: VariableOrOptional[StorageModeParam]
    """
    The default storage mode of the Power BI model
    """

    workspace_name: VariableOrOptional[str]
    """
    The name of the Power BI workspace of the model
    """


PowerBiModelParam = PowerBiModelDict | PowerBiModel



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr, VariableOrOptional

if TYPE_CHECKING:
    from typing_extensions import Self

    from databricks.bundles.jobs._models.task import Task, TaskParam


@dataclass(kw_only=True)
class ForEachTask:
    """"""

    inputs: VariableOr[str]
    """
    Array for task to iterate on. This can be a JSON string or a reference to
    an array parameter.
    """

    task: VariableOr["Task"]
    """
    Configuration for the task that will be run for each element in the array
    """

    concurrency: VariableOrOptional[int] = None
    """
    An optional maximum allowed number of concurrent runs of the task.
    Set this value if you want to be able to execute multiple runs of the task concurrently.
    """

    @classmethod
    def from_dict(cls, value: "ForEachTaskDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "ForEachTaskDict":
        return _transform_to_json_value(self)  # type:ignore


class ForEachTaskDict(TypedDict, total=False):
    """"""

    inputs: VariableOr[str]
    """
    Array for task to iterate on. This can be a JSON string or a reference to
    an array parameter.
    """

    task: VariableOr["TaskParam"]
    """
    Configuration for the task that will be run for each element in the array
    """

    concurrency: VariableOrOptional[int]
    """
    An optional maximum allowed number of concurrent runs of the task.
    Set this value if you want to be able to execute multiple runs of the task concurrently.
    """


ForEachTaskParam = ForEachTaskDict | ForEachTask



from enum import Enum
from typing import Literal


class IngestionSourceType(Enum):
    MYSQL = "MYSQL"
    POSTGRESQL = "POSTGRESQL"
    SQLSERVER = "SQLSERVER"
    SALESFORCE = "SALESFORCE"
    NETSUITE = "NETSUITE"
    WORKDAY_RAAS = "WORKDAY_RAAS"
    GA4_RAW_DATA = "GA4_RAW_DATA"
    SERVICENOW = "SERVICENOW"
    MANAGED_POSTGRESQL = "MANAGED_POSTGRESQL"
    ORACLE = "ORACLE"
    TERADATA = "TERADATA"
    SHAREPOINT = "SHAREPOINT"
    DYNAMICS365 = "DYNAMICS365"


IngestionSourceTypeParam = (
    Literal[
        "MYSQL",
        "POSTGRESQL",
        "SQLSERVER",
        "SALESFORCE",
        "NETSUITE",
        "WORKDAY_RAAS",
        "GA4_RAW_DATA",
        "SERVICENOW",
        "MANAGED_POSTGRESQL",
        "ORACLE",
        "TERADATA",
        "SHAREPOINT",
        "DYNAMICS365",
    ]
    | IngestionSourceType
)



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class DbfsStorageInfo:
    """
    A storage location in DBFS
    """

    destination: VariableOr[str]
    """
    dbfs destination, e.g. `dbfs:/my/path`
    """

    @classmethod
    def from_dict(cls, value: "DbfsStorageInfoDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "DbfsStorageInfoDict":
        return _transform_to_json_value(self)  # type:ignore


class DbfsStorageInfoDict(TypedDict, total=False):
    """"""

    destination: VariableOr[str]
    """
    dbfs destination, e.g. `dbfs:/my/path`
    """


DbfsStorageInfoParam = DbfsStorageInfoDict | DbfsStorageInfo



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrList, VariableOrOptional
from databricks.bundles.jobs._models.subscription_subscriber import (
    SubscriptionSubscriber,
    SubscriptionSubscriberParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class Subscription:
    """"""

    custom_subject: VariableOrOptional[str] = None
    """
    Optional: Allows users to specify a custom subject line on the email sent
    to subscribers.
    """

    paused: VariableOrOptional[bool] = None
    """
    When true, the subscription will not send emails.
    """

    subscribers: VariableOrList[SubscriptionSubscriber] = field(default_factory=list)

    @classmethod
    def from_dict(cls, value: "SubscriptionDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "SubscriptionDict":
        return _transform_to_json_value(self)  # type:ignore


class SubscriptionDict(TypedDict, total=False):
    """"""

    custom_subject: VariableOrOptional[str]
    """
    Optional: Allows users to specify a custom subject line on the email sent
    to subscribers.
    """

    paused: VariableOrOptional[bool]
    """
    When true, the subscription will not send emails.
    """

    subscribers: VariableOrList[SubscriptionSubscriberParam]


SubscriptionParam = SubscriptionDict | Subscription



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._resource import Resource
from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import (
    VariableOrDict,
    VariableOrList,
    VariableOrOptional,
)
from databricks.bundles.jobs._models.continuous import (
    Continuous,
    ContinuousParam,
)
from databricks.bundles.jobs._models.cron_schedule import (
    CronSchedule,
    CronScheduleParam,
)
from databricks.bundles.jobs._models.git_source import GitSource, GitSourceParam
from databricks.bundles.jobs._models.job_cluster import JobCluster, JobClusterParam
from databricks.bundles.jobs._models.job_email_notifications import (
    JobEmailNotifications,
    JobEmailNotificationsParam,
)
from databricks.bundles.jobs._models.job_environment import (
    JobEnvironment,
    JobEnvironmentParam,
)
from databricks.bundles.jobs._models.job_notification_settings import (
    JobNotificationSettings,
    JobNotificationSettingsParam,
)
from databricks.bundles.jobs._models.job_parameter_definition import (
    JobParameterDefinition,
    JobParameterDefinitionParam,
)
from databricks.bundles.jobs._models.job_permission import (
    JobPermission,
    JobPermissionParam,
)
from databricks.bundles.jobs._models.job_run_as import JobRunAs, JobRunAsParam
from databricks.bundles.jobs._models.jobs_health_rules import (
    JobsHealthRules,
    JobsHealthRulesParam,
)
from databricks.bundles.jobs._models.performance_target import (
    PerformanceTarget,
    PerformanceTargetParam,
)
from databricks.bundles.jobs._models.queue_settings import (
    QueueSettings,
    QueueSettingsParam,
)
from databricks.bundles.jobs._models.task import Task, TaskParam
from databricks.bundles.jobs._models.trigger_settings import (
    TriggerSettings,
    TriggerSettingsParam,
)
from databricks.bundles.jobs._models.webhook_notifications import (
    WebhookNotifications,
    WebhookNotificationsParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class Job(Resource):
    """"""

    budget_policy_id: VariableOrOptional[str] = None
    """
    The id of the user specified budget policy to use for this job.
    If not specified, a default budget policy may be applied when creating or modifying the job.
    See `effective_budget_policy_id` for the budget policy used by this workload.
    """

    continuous: VariableOrOptional[Continuous] = None
    """
    An optional continuous property for this job. The continuous property will ensure that there is always one run executing. Only one of `schedule` and `continuous` can be used.
    """

    description: VariableOrOptional[str] = None
    """
    An optional description for the job. The maximum length is 27700 characters in UTF-8 encoding.
    """

    email_notifications: VariableOrOptional[JobEmailNotifications] = None
    """
    An optional set of email addresses that is notified when runs of this job begin or complete as well as when this job is deleted.
    """

    environments: VariableOrList[JobEnvironment] = field(default_factory=list)
    """
    A list of task execution environment specifications that can be referenced by serverless tasks of this job.
    An environment is required to be present for serverless tasks.
    For serverless notebook tasks, the environment is accessible in the notebook environment panel.
    For other serverless tasks, the task environment is required to be specified using environment_key in the task settings.
    """

    git_source: VariableOrOptional[GitSource] = None
    """
    An optional specification for a remote Git repository containing the source code used by tasks. Version-controlled source code is supported by notebook, dbt, Python script, and SQL File tasks.
    
    If `git_source` is set, these tasks retrieve the file from the remote repository by default. However, this behavior can be overridden by setting `source` to `WORKSPACE` on the task.
    
    Note: dbt and SQL File tasks support only version-controlled sources. If dbt or SQL File tasks are used, `git_source` must be defined on the job.
    """

    health: VariableOrOptional[JobsHealthRules] = None

    job_clusters: VariableOrList[JobCluster] = field(default_factory=list)
    """
    A list of job cluster specifications that can be shared and reused by tasks of this job. Libraries cannot be declared in a shared job cluster. You must declare dependent libraries in task settings.
    """

    max_concurrent_runs: VariableOrOptional[int] = None
    """
    An optional maximum allowed number of concurrent runs of the job.
    Set this value if you want to be able to execute multiple runs of the same job concurrently.
    This is useful for example if you trigger your job on a frequent schedule and want to allow consecutive runs to overlap with each other, or if you want to trigger multiple runs which differ by their input parameters.
    This setting affects only new runs. For example, suppose the job’s concurrency is 4 and there are 4 concurrent active runs. Then setting the concurrency to 3 won’t kill any of the active runs.
    However, from then on, new runs are skipped unless there are fewer than 3 active runs.
    This value cannot exceed 1000. Setting this value to `0` causes all new runs to be skipped.
    """

    name: VariableOrOptional[str] = None
    """
    An optional name for the job. The maximum length is 4096 bytes in UTF-8 encoding.
    """

    notification_settings: VariableOrOptional[JobNotificationSettings] = None
    """
    Optional notification settings that are used when sending notifications to each of the `email_notifications` and `webhook_notifications` for this job.
    """

    parameters: VariableOrList[JobParameterDefinition] = field(default_factory=list)
    """
    Job-level parameter definitions
    """

    performance_target: VariableOrOptional[PerformanceTarget] = None
    """
    The performance mode on a serverless job. This field determines the level of compute performance or cost-efficiency for the run.
    
    * `STANDARD`: Enables cost-efficient execution of serverless workloads.
    * `PERFORMANCE_OPTIMIZED`: Prioritizes fast startup and execution times through rapid scaling and optimized cluster performance.
    """

    permissions: VariableOrList[JobPermission] = field(default_factory=list)

    queue: VariableOrOptional[QueueSettings] = None
    """
    The queue settings of the job.
    """

    run_as: VariableOrOptional[JobRunAs] = None

    schedule: VariableOrOptional[CronSchedule] = None
    """
    An optional periodic schedule for this job. The default behavior is that the job only runs when triggered by clicking “Run Now” in the Jobs UI or sending an API request to `runNow`.
    """

    tags: VariableOrDict[str] = field(default_factory=dict)
    """
    A map of tags associated with the job. These are forwarded to the cluster as cluster tags for jobs clusters, and are subject to the same limitations as cluster tags. A maximum of 25 tags can be added to the job.
    """

    tasks: VariableOrList[Task] = field(default_factory=list)
    """
    A list of task specifications to be executed by this job.
    It supports up to 1000 elements in write endpoints (:method:jobs/create, :method:jobs/reset, :method:jobs/update, :method:jobs/submit).
    Read endpoints return only 100 tasks. If more than 100 tasks are available, you can paginate through them using :method:jobs/get. Use the `next_page_token` field at the object root to determine if more results are available.
    """

    timeout_seconds: VariableOrOptional[int] = None
    """
    An optional timeout applied to each run of this job. A value of `0` means no timeout.
    """

    trigger: VariableOrOptional[TriggerSettings] = None
    """
    A configuration to trigger a run when certain conditions are met. The default behavior is that the job runs only when triggered by clicking “Run Now” in the Jobs UI or sending an API request to `runNow`.
    """

    webhook_notifications: VariableOrOptional[WebhookNotifications] = None
    """
    A collection of system notification IDs to notify when runs of this job begin or complete.
    """

    @classmethod
    def from_dict(cls, value: "JobDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "JobDict":
        return _transform_to_json_value(self)  # type:ignore


class JobDict(TypedDict, total=False):
    """"""

    budget_policy_id: VariableOrOptional[str]
    """
    The id of the user specified budget policy to use for this job.
    If not specified, a default budget policy may be applied when creating or modifying the job.
    See `effective_budget_policy_id` for the budget policy used by this workload.
    """

    continuous: VariableOrOptional[ContinuousParam]
    """
    An optional continuous property for this job. The continuous property will ensure that there is always one run executing. Only one of `schedule` and `continuous` can be used.
    """

    description: VariableOrOptional[str]
    """
    An optional description for the job. The maximum length is 27700 characters in UTF-8 encoding.
    """

    email_notifications: VariableOrOptional[JobEmailNotificationsParam]
    """
    An optional set of email addresses that is notified when runs of this job begin or complete as well as when this job is deleted.
    """

    environments: VariableOrList[JobEnvironmentParam]
    """
    A list of task execution environment specifications that can be referenced by serverless tasks of this job.
    An environment is required to be present for serverless tasks.
    For serverless notebook tasks, the environment is accessible in the notebook environment panel.
    For other serverless tasks, the task environment is required to be specified using environment_key in the task settings.
    """

    git_source: VariableOrOptional[GitSourceParam]
    """
    An optional specification for a remote Git repository containing the source code used by tasks. Version-controlled source code is supported by notebook, dbt, Python script, and SQL File tasks.
    
    If `git_source` is set, these tasks retrieve the file from the remote repository by default. However, this behavior can be overridden by setting `source` to `WORKSPACE` on the task.
    
    Note: dbt and SQL File tasks support only version-controlled sources. If dbt or SQL File tasks are used, `git_source` must be defined on the job.
    """

    health: VariableOrOptional[JobsHealthRulesParam]

    job_clusters: VariableOrList[JobClusterParam]
    """
    A list of job cluster specifications that can be shared and reused by tasks of this job. Libraries cannot be declared in a shared job cluster. You must declare dependent libraries in task settings.
    """

    max_concurrent_runs: VariableOrOptional[int]
    """
    An optional maximum allowed number of concurrent runs of the job.
    Set this value if you want to be able to execute multiple runs of the same job concurrently.
    This is useful for example if you trigger your job on a frequent schedule and want to allow consecutive runs to overlap with each other, or if you want to trigger multiple runs which differ by their input parameters.
    This setting affects only new runs. For example, suppose the job’s concurrency is 4 and there are 4 concurrent active runs. Then setting the concurrency to 3 won’t kill any of the active runs.
    However, from then on, new runs are skipped unless there are fewer than 3 active runs.
    This value cannot exceed 1000. Setting this value to `0` causes all new runs to be skipped.
    """

    name: VariableOrOptional[str]
    """
    An optional name for the job. The maximum length is 4096 bytes in UTF-8 encoding.
    """

    notification_settings: VariableOrOptional[JobNotificationSettingsParam]
    """
    Optional notification settings that are used when sending notifications to each of the `email_notifications` and `webhook_notifications` for this job.
    """

    parameters: VariableOrList[JobParameterDefinitionParam]
    """
    Job-level parameter definitions
    """

    performance_target: VariableOrOptional[PerformanceTargetParam]
    """
    The performance mode on a serverless job. This field determines the level of compute performance or cost-efficiency for the run.
    
    * `STANDARD`: Enables cost-efficient execution of serverless workloads.
    * `PERFORMANCE_OPTIMIZED`: Prioritizes fast startup and execution times through rapid scaling and optimized cluster performance.
    """

    permissions: VariableOrList[JobPermissionParam]

    queue: VariableOrOptional[QueueSettingsParam]
    """
    The queue settings of the job.
    """

    run_as: VariableOrOptional[JobRunAsParam]

    schedule: VariableOrOptional[CronScheduleParam]
    """
    An optional periodic schedule for this job. The default behavior is that the job only runs when triggered by clicking “Run Now” in the Jobs UI or sending an API request to `runNow`.
    """

    tags: VariableOrDict[str]
    """
    A map of tags associated with the job. These are forwarded to the cluster as cluster tags for jobs clusters, and are subject to the same limitations as cluster tags. A maximum of 25 tags can be added to the job.
    """

    tasks: VariableOrList[TaskParam]
    """
    A list of task specifications to be executed by this job.
    It supports up to 1000 elements in write endpoints (:method:jobs/create, :method:jobs/reset, :method:jobs/update, :method:jobs/submit).
    Read endpoints return only 100 tasks. If more than 100 tasks are available, you can paginate through them using :method:jobs/get. Use the `next_page_token` field at the object root to determine if more results are available.
    """

    timeout_seconds: VariableOrOptional[int]
    """
    An optional timeout applied to each run of this job. A value of `0` means no timeout.
    """

    trigger: VariableOrOptional[TriggerSettingsParam]
    """
    A configuration to trigger a run when certain conditions are met. The default behavior is that the job runs only when triggered by clicking “Run Now” in the Jobs UI or sending an API request to `runNow`.
    """

    webhook_notifications: VariableOrOptional[WebhookNotificationsParam]
    """
    A collection of system notification IDs to notify when runs of this job begin or complete.
    """


JobParam = JobDict | Job



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional
from databricks.bundles.jobs._models.file_arrival_trigger_configuration import (
    FileArrivalTriggerConfiguration,
    FileArrivalTriggerConfigurationParam,
)
from databricks.bundles.jobs._models.pause_status import PauseStatus, PauseStatusParam
from databricks.bundles.jobs._models.periodic_trigger_configuration import (
    PeriodicTriggerConfiguration,
    PeriodicTriggerConfigurationParam,
)
from databricks.bundles.jobs._models.table_update_trigger_configuration import (
    TableUpdateTriggerConfiguration,
    TableUpdateTriggerConfigurationParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class TriggerSettings:
    """"""

    file_arrival: VariableOrOptional[FileArrivalTriggerConfiguration] = None
    """
    File arrival trigger settings.
    """

    pause_status: VariableOrOptional[PauseStatus] = None
    """
    Whether this trigger is paused or not.
    """

    periodic: VariableOrOptional[PeriodicTriggerConfiguration] = None
    """
    Periodic trigger settings.
    """

    table_update: VariableOrOptional[TableUpdateTriggerConfiguration] = None
    """
    :meta private: [EXPERIMENTAL]
    """

    @classmethod
    def from_dict(cls, value: "TriggerSettingsDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "TriggerSettingsDict":
        return _transform_to_json_value(self)  # type:ignore


class TriggerSettingsDict(TypedDict, total=False):
    """"""

    file_arrival: VariableOrOptional[FileArrivalTriggerConfigurationParam]
    """
    File arrival trigger settings.
    """

    pause_status: VariableOrOptional[PauseStatusParam]
    """
    Whether this trigger is paused or not.
    """

    periodic: VariableOrOptional[PeriodicTriggerConfigurationParam]
    """
    Periodic trigger settings.
    """

    table_update: VariableOrOptional[TableUpdateTriggerConfigurationParam]
    """
    :meta private: [EXPERIMENTAL]
    """


TriggerSettingsParam = TriggerSettingsDict | TriggerSettings



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr, VariableOrOptional

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class S3StorageInfo:
    """
    A storage location in Amazon S3
    """

    destination: VariableOr[str]
    """
    S3 destination, e.g. `s3://my-bucket/some-prefix` Note that logs will be delivered using
    cluster iam role, please make sure you set cluster iam role and the role has write access to the
    destination. Please also note that you cannot use AWS keys to deliver logs.
    """

    canned_acl: VariableOrOptional[str] = None
    """
    (Optional) Set canned access control list for the logs, e.g. `bucket-owner-full-control`.
    If `canned_cal` is set, please make sure the cluster iam role has `s3:PutObjectAcl` permission on
    the destination bucket and prefix. The full list of possible canned acl can be found at
    http://docs.aws.amazon.com/AmazonS3/latest/dev/acl-overview.html#canned-acl.
    Please also note that by default only the object owner gets full controls. If you are using cross account
    role for writing data, you may want to set `bucket-owner-full-control` to make bucket owner able to
    read the logs.
    """

    enable_encryption: VariableOrOptional[bool] = None
    """
    (Optional) Flag to enable server side encryption, `false` by default.
    """

    encryption_type: VariableOrOptional[str] = None
    """
    (Optional) The encryption type, it could be `sse-s3` or `sse-kms`. It will be used only when
    encryption is enabled and the default type is `sse-s3`.
    """

    endpoint: VariableOrOptional[str] = None
    """
    S3 endpoint, e.g. `https://s3-us-west-2.amazonaws.com`. Either region or endpoint needs to be set.
    If both are set, endpoint will be used.
    """

    kms_key: VariableOrOptional[str] = None
    """
    (Optional) Kms key which will be used if encryption is enabled and encryption type is set to `sse-kms`.
    """

    region: VariableOrOptional[str] = None
    """
    S3 region, e.g. `us-west-2`. Either region or endpoint needs to be set. If both are set,
    endpoint will be used.
    """

    @classmethod
    def from_dict(cls, value: "S3StorageInfoDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "S3StorageInfoDict":
        return _transform_to_json_value(self)  # type:ignore


class S3StorageInfoDict(TypedDict, total=False):
    """"""

    destination: VariableOr[str]
    """
    S3 destination, e.g. `s3://my-bucket/some-prefix` Note that logs will be delivered using
    cluster iam role, please make sure you set cluster iam role and the role has write access to the
    destination. Please also note that you cannot use AWS keys to deliver logs.
    """

    canned_acl: VariableOrOptional[str]
    """
    (Optional) Set canned access control list for the logs, e.g. `bucket-owner-full-control`.
    If `canned_cal` is set, please make sure the cluster iam role has `s3:PutObjectAcl` permission on
    the destination bucket and prefix. The full list of possible canned acl can be found at
    http://docs.aws.amazon.com/AmazonS3/latest/dev/acl-overview.html#canned-acl.
    Please also note that by default only the object owner gets full controls. If you are using cross account
    role for writing data, you may want to set `bucket-owner-full-control` to make bucket owner able to
    read the logs.
    """

    enable_encryption: VariableOrOptional[bool]
    """
    (Optional) Flag to enable server side encryption, `false` by default.
    """

    encryption_type: VariableOrOptional[str]
    """
    (Optional) The encryption type, it could be `sse-s3` or `sse-kms`. It will be used only when
    encryption is enabled and the default type is `sse-s3`.
    """

    endpoint: VariableOrOptional[str]
    """
    S3 endpoint, e.g. `https://s3-us-west-2.amazonaws.com`. Either region or endpoint needs to be set.
    If both are set, endpoint will be used.
    """

    kms_key: VariableOrOptional[str]
    """
    (Optional) Kms key which will be used if encryption is enabled and encryption type is set to `sse-kms`.
    """

    region: VariableOrOptional[str]
    """
    S3 region, e.g. `us-west-2`. Either region or endpoint needs to be set. If both are set,
    endpoint will be used.
    """


S3StorageInfoParam = S3StorageInfoDict | S3StorageInfo



from enum import Enum
from typing import Literal


class EbsVolumeType(Enum):
    """
    All EBS volume types that Databricks supports.
    See https://aws.amazon.com/ebs/details/ for details.
    """

    GENERAL_PURPOSE_SSD = "GENERAL_PURPOSE_SSD"
    THROUGHPUT_OPTIMIZED_HDD = "THROUGHPUT_OPTIMIZED_HDD"


EbsVolumeTypeParam = (
    Literal["GENERAL_PURPOSE_SSD", "THROUGHPUT_OPTIMIZED_HDD"] | EbsVolumeType
)



from enum import Enum
from typing import Literal


class DataSecurityMode(Enum):
    """
    Data security mode decides what data governance model to use when accessing data
    from a cluster.

    The following modes can only be used when `kind = CLASSIC_PREVIEW`.
    * `DATA_SECURITY_MODE_AUTO`: Databricks will choose the most appropriate access mode depending on your compute configuration.
    * `DATA_SECURITY_MODE_STANDARD`: Alias for `USER_ISOLATION`.
    * `DATA_SECURITY_MODE_DEDICATED`: Alias for `SINGLE_USER`.

    The following modes can be used regardless of `kind`.
    * `NONE`: No security isolation for multiple users sharing the cluster. Data governance features are not available in this mode.
    * `SINGLE_USER`: A secure cluster that can only be exclusively used by a single user specified in `single_user_name`. Most programming languages, cluster features and data governance features are available in this mode.
    * `USER_ISOLATION`: A secure cluster that can be shared by multiple users. Cluster users are fully isolated so that they cannot see each other's data and credentials. Most data governance features are supported in this mode. But programming languages and cluster features might be limited.

    The following modes are deprecated starting with Databricks Runtime 15.0 and
    will be removed for future Databricks Runtime versions:

    * `LEGACY_TABLE_ACL`: This mode is for users migrating from legacy Table ACL clusters.
    * `LEGACY_PASSTHROUGH`: This mode is for users migrating from legacy Passthrough on high concurrency clusters.
    * `LEGACY_SINGLE_USER`: This mode is for users migrating from legacy Passthrough on standard clusters.
    * `LEGACY_SINGLE_USER_STANDARD`: This mode provides a way that doesn’t have UC nor passthrough enabled.
    """

    NONE = "NONE"
    SINGLE_USER = "SINGLE_USER"
    USER_ISOLATION = "USER_ISOLATION"
    LEGACY_TABLE_ACL = "LEGACY_TABLE_ACL"
    LEGACY_PASSTHROUGH = "LEGACY_PASSTHROUGH"
    LEGACY_SINGLE_USER = "LEGACY_SINGLE_USER"
    LEGACY_SINGLE_USER_STANDARD = "LEGACY_SINGLE_USER_STANDARD"
    DATA_SECURITY_MODE_STANDARD = "DATA_SECURITY_MODE_STANDARD"
    DATA_SECURITY_MODE_DEDICATED = "DATA_SECURITY_MODE_DEDICATED"
    DATA_SECURITY_MODE_AUTO = "DATA_SECURITY_MODE_AUTO"


DataSecurityModeParam = (
    Literal[
        "NONE",
        "SINGLE_USER",
        "USER_ISOLATION",
        "LEGACY_TABLE_ACL",
        "LEGACY_PASSTHROUGH",
        "LEGACY_SINGLE_USER",
        "LEGACY_SINGLE_USER_STANDARD",
        "DATA_SECURITY_MODE_STANDARD",
        "DATA_SECURITY_MODE_DEDICATED",
        "DATA_SECURITY_MODE_AUTO",
    ]
    | DataSecurityMode
)



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrList, VariableOrOptional
from databricks.bundles.pipelines._models.table_specific_config_scd_type import (
    TableSpecificConfigScdType,
    TableSpecificConfigScdTypeParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class TableSpecificConfig:
    """"""

    exclude_columns: VariableOrList[str] = field(default_factory=list)
    """
    A list of column names to be excluded for the ingestion.
    When not specified, include_columns fully controls what columns to be ingested.
    When specified, all other columns including future ones will be automatically included for ingestion.
    This field in mutually exclusive with `include_columns`.
    """

    include_columns: VariableOrList[str] = field(default_factory=list)
    """
    A list of column names to be included for the ingestion.
    When not specified, all columns except ones in exclude_columns will be included. Future
    columns will be automatically included.
    When specified, all other future columns will be automatically excluded from ingestion.
    This field in mutually exclusive with `exclude_columns`.
    """

    primary_keys: VariableOrList[str] = field(default_factory=list)
    """
    The primary key of the table used to apply changes.
    """

    salesforce_include_formula_fields: VariableOrOptional[bool] = None
    """
    :meta private: [EXPERIMENTAL]
    
    If true, formula fields defined in the table are included in the ingestion. This setting is only valid for the Salesforce connector
    """

    scd_type: VariableOrOptional[TableSpecificConfigScdType] = None
    """
    :meta private: [EXPERIMENTAL]
    
    The SCD type to use to ingest the table.
    """

    sequence_by: VariableOrList[str] = field(default_factory=list)
    """
    The column names specifying the logical order of events in the source data. Delta Live Tables uses this sequencing to handle change events that arrive out of order.
    """

    @classmethod
    def from_dict(cls, value: "TableSpecificConfigDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "TableSpecificConfigDict":
        return _transform_to_json_value(self)  # type:ignore


class TableSpecificConfigDict(TypedDict, total=False):
    """"""

    exclude_columns: VariableOrList[str]
    """
    A list of column names to be excluded for the ingestion.
    When not specified, include_columns fully controls what columns to be ingested.
    When specified, all other columns including future ones will be automatically included for ingestion.
    This field in mutually exclusive with `include_columns`.
    """

    include_columns: VariableOrList[str]
    """
    A list of column names to be included for the ingestion.
    When not specified, all columns except ones in exclude_columns will be included. Future
    columns will be automatically included.
    When specified, all other future columns will be automatically excluded from ingestion.
    This field in mutually exclusive with `exclude_columns`.
    """

    primary_keys: VariableOrList[str]
    """
    The primary key of the table used to apply changes.
    """

    salesforce_include_formula_fields: VariableOrOptional[bool]
    """
    :meta private: [EXPERIMENTAL]
    
    If true, formula fields defined in the table are included in the ingestion. This setting is only valid for the Salesforce connector
    """

    scd_type: VariableOrOptional[TableSpecificConfigScdTypeParam]
    """
    :meta private: [EXPERIMENTAL]
    
    The SCD type to use to ingest the table.
    """

    sequence_by: VariableOrList[str]
    """
    The column names specifying the logical order of events in the source data. Delta Live Tables uses this sequencing to handle change events that arrive out of order.
    """


TableSpecificConfigParam = TableSpecificConfigDict | TableSpecificConfig



from enum import Enum
from typing import Literal


class RuntimeEngine(Enum):
    NULL = "NULL"
    STANDARD = "STANDARD"
    PHOTON = "PHOTON"


RuntimeEngineParam = Literal["NULL", "STANDARD", "PHOTON"] | RuntimeEngine



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import (
    VariableOr,
    VariableOrList,
    VariableOrOptional,
)
from databricks.bundles.jobs._models.source import Source, SourceParam

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class SparkPythonTask:
    """"""

    python_file: VariableOr[str]
    """
    The Python file to be executed. Cloud file URIs (such as dbfs:/, s3:/, adls:/, gcs:/) and workspace paths are supported. For python files stored in the Databricks workspace, the path must be absolute and begin with `/`. For files stored in a remote repository, the path must be relative. This field is required.
    """

    parameters: VariableOrList[str] = field(default_factory=list)
    """
    Command line parameters passed to the Python file.
    
    Use [Task parameter variables](https://docs.databricks.com/jobs.html#parameter-variables) to set parameters containing information about job runs.
    """

    source: VariableOrOptional[Source] = None
    """
    Optional location type of the Python file. When set to `WORKSPACE` or not specified, the file will be retrieved from the local
    Databricks workspace or cloud location (if the `python_file` has a URI format). When set to `GIT`,
    the Python file will be retrieved from a Git repository defined in `git_source`.
    
    * `WORKSPACE`: The Python file is located in a Databricks workspace or at a cloud filesystem URI.
    * `GIT`: The Python file is located in a remote Git repository.
    """

    @classmethod
    def from_dict(cls, value: "SparkPythonTaskDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "SparkPythonTaskDict":
        return _transform_to_json_value(self)  # type:ignore


class SparkPythonTaskDict(TypedDict, total=False):
    """"""

    python_file: VariableOr[str]
    """
    The Python file to be executed. Cloud file URIs (such as dbfs:/, s3:/, adls:/, gcs:/) and workspace paths are supported. For python files stored in the Databricks workspace, the path must be absolute and begin with `/`. For files stored in a remote repository, the path must be relative. This field is required.
    """

    parameters: VariableOrList[str]
    """
    Command line parameters passed to the Python file.
    
    Use [Task parameter variables](https://docs.databricks.com/jobs.html#parameter-variables) to set parameters containing information about job runs.
    """

    source: VariableOrOptional[SourceParam]
    """
    Optional location type of the Python file. When set to `WORKSPACE` or not specified, the file will be retrieved from the local
    Databricks workspace or cloud location (if the `python_file` has a URI format). When set to `GIT`,
    the Python file will be retrieved from a Git repository defined in `git_source`.
    
    * `WORKSPACE`: The Python file is located in a Databricks workspace or at a cloud filesystem URI.
    * `GIT`: The Python file is located in a remote Git repository.
    """


SparkPythonTaskParam = SparkPythonTaskDict | SparkPythonTask



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.compute._models.maven_library import (
    MavenLibrary,
    MavenLibraryParam,
)
from databricks.bundles.compute._models.python_py_pi_library import (
    PythonPyPiLibrary,
    PythonPyPiLibraryParam,
)
from databricks.bundles.compute._models.r_cran_library import (
    RCranLibrary,
    RCranLibraryParam,
)
from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class Library:
    """"""

    cran: VariableOrOptional[RCranLibrary] = None
    """
    Specification of a CRAN library to be installed as part of the library
    """

    jar: VariableOrOptional[str] = None
    """
    URI of the JAR library to install. Supported URIs include Workspace paths, Unity Catalog Volumes paths, and S3 URIs.
    For example: `{ "jar": "/Workspace/path/to/library.jar" }`, `{ "jar" : "/Volumes/path/to/library.jar" }` or
    `{ "jar": "s3://my-bucket/library.jar" }`.
    If S3 is used, please make sure the cluster has read access on the library. You may need to
    launch the cluster with an IAM role to access the S3 URI.
    """

    maven: VariableOrOptional[MavenLibrary] = None
    """
    Specification of a maven library to be installed. For example:
    `{ "coordinates": "org.jsoup:jsoup:1.7.2" }`
    """

    pypi: VariableOrOptional[PythonPyPiLibrary] = None
    """
    Specification of a PyPi library to be installed. For example:
    `{ "package": "simplejson" }`
    """

    requirements: VariableOrOptional[str] = None
    """
    URI of the requirements.txt file to install. Only Workspace paths and Unity Catalog Volumes paths are supported.
    For example: `{ "requirements": "/Workspace/path/to/requirements.txt" }` or `{ "requirements" : "/Volumes/path/to/requirements.txt" }`
    """

    whl: VariableOrOptional[str] = None
    """
    URI of the wheel library to install. Supported URIs include Workspace paths, Unity Catalog Volumes paths, and S3 URIs.
    For example: `{ "whl": "/Workspace/path/to/library.whl" }`, `{ "whl" : "/Volumes/path/to/library.whl" }` or
    `{ "whl": "s3://my-bucket/library.whl" }`.
    If S3 is used, please make sure the cluster has read access on the library. You may need to
    launch the cluster with an IAM role to access the S3 URI.
    """

    @classmethod
    def from_dict(cls, value: "LibraryDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "LibraryDict":
        return _transform_to_json_value(self)  # type:ignore


class LibraryDict(TypedDict, total=False):
    """"""

    cran: VariableOrOptional[RCranLibraryParam]
    """
    Specification of a CRAN library to be installed as part of the library
    """

    jar: VariableOrOptional[str]
    """
    URI of the JAR library to install. Supported URIs include Workspace paths, Unity Catalog Volumes paths, and S3 URIs.
    For example: `{ "jar": "/Workspace/path/to/library.jar" }`, `{ "jar" : "/Volumes/path/to/library.jar" }` or
    `{ "jar": "s3://my-bucket/library.jar" }`.
    If S3 is used, please make sure the cluster has read access on the library. You may need to
    launch the cluster with an IAM role to access the S3 URI.
    """

    maven: VariableOrOptional[MavenLibraryParam]
    """
    Specification of a maven library to be installed. For example:
    `{ "coordinates": "org.jsoup:jsoup:1.7.2" }`
    """

    pypi: VariableOrOptional[PythonPyPiLibraryParam]
    """
    Specification of a PyPi library to be installed. For example:
    `{ "package": "simplejson" }`
    """

    requirements: VariableOrOptional[str]
    """
    URI of the requirements.txt file to install. Only Workspace paths and Unity Catalog Volumes paths are supported.
    For example: `{ "requirements": "/Workspace/path/to/requirements.txt" }` or `{ "requirements" : "/Volumes/path/to/requirements.txt" }`
    """

    whl: VariableOrOptional[str]
    """
    URI of the wheel library to install. Supported URIs include Workspace paths, Unity Catalog Volumes paths, and S3 URIs.
    For example: `{ "whl": "/Workspace/path/to/library.whl" }`, `{ "whl" : "/Volumes/path/to/library.whl" }` or
    `{ "whl": "s3://my-bucket/library.whl" }`.
    If S3 is used, please make sure the cluster has read access on the library. You may need to
    launch the cluster with an IAM role to access the S3 URI.
    """


LibraryParam = LibraryDict | Library



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr, VariableOrOptional
from databricks.bundles.jobs._models.git_provider import GitProvider, GitProviderParam

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class GitSource:
    """
    An optional specification for a remote Git repository containing the source code used by tasks. Version-controlled source code is supported by notebook, dbt, Python script, and SQL File tasks.

    If `git_source` is set, these tasks retrieve the file from the remote repository by default. However, this behavior can be overridden by setting `source` to `WORKSPACE` on the task.

    Note: dbt and SQL File tasks support only version-controlled sources. If dbt or SQL File tasks are used, `git_source` must be defined on the job.
    """

    git_provider: VariableOr[GitProvider]
    """
    Unique identifier of the service used to host the Git repository. The value is case insensitive.
    """

    git_url: VariableOr[str]
    """
    URL of the repository to be cloned by this job.
    """

    git_branch: VariableOrOptional[str] = None
    """
    Name of the branch to be checked out and used by this job. This field cannot be specified in conjunction with git_tag or git_commit.
    """

    git_commit: VariableOrOptional[str] = None
    """
    Commit to be checked out and used by this job. This field cannot be specified in conjunction with git_branch or git_tag.
    """

    git_tag: VariableOrOptional[str] = None
    """
    Name of the tag to be checked out and used by this job. This field cannot be specified in conjunction with git_branch or git_commit.
    """

    @classmethod
    def from_dict(cls, value: "GitSourceDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "GitSourceDict":
        return _transform_to_json_value(self)  # type:ignore


class GitSourceDict(TypedDict, total=False):
    """"""

    git_provider: VariableOr[GitProviderParam]
    """
    Unique identifier of the service used to host the Git repository. The value is case insensitive.
    """

    git_url: VariableOr[str]
    """
    URL of the repository to be cloned by this job.
    """

    git_branch: VariableOrOptional[str]
    """
    Name of the branch to be checked out and used by this job. This field cannot be specified in conjunction with git_tag or git_commit.
    """

    git_commit: VariableOrOptional[str]
    """
    Commit to be checked out and used by this job. This field cannot be specified in conjunction with git_branch or git_tag.
    """

    git_tag: VariableOrOptional[str]
    """
    Name of the tag to be checked out and used by this job. This field cannot be specified in conjunction with git_branch or git_commit.
    """


GitSourceParam = GitSourceDict | GitSource



__all__ = [
    "Bundle",
    "Diagnostic",
    "Diagnostics",
    "Location",
    "Resource",
    "ResourceMutator",
    "Resources",
    "Severity",
    "Variable",
    "VariableOr",
    "VariableOrDict",
    "VariableOrList",
    "VariableOrOptional",
    "job_mutator",
    "pipeline_mutator",
    "load_resources_from_current_package_module",
    "load_resources_from_module",
    "load_resources_from_modules",
    "load_resources_from_package_module",
    "variables",
]

from databricks.bundles.core._bundle import Bundle
from databricks.bundles.core._diagnostics import (
    Diagnostic,
    Diagnostics,
    Severity,
)
from databricks.bundles.core._load import (
    load_resources_from_current_package_module,
    load_resources_from_module,
    load_resources_from_modules,
    load_resources_from_package_module,
)
from databricks.bundles.core._location import Location
from databricks.bundles.core._resource import Resource
from databricks.bundles.core._resource_mutator import (
    ResourceMutator,
    job_mutator,
    pipeline_mutator,
)
from databricks.bundles.core._resources import Resources
from databricks.bundles.core._variable import (
    Variable,
    VariableOr,
    VariableOrDict,
    VariableOrList,
    VariableOrOptional,
    variables,
)



from typing import TYPE_CHECKING, Optional

from databricks.bundles.core._diagnostics import Diagnostics
from databricks.bundles.core._location import Location
from databricks.bundles.core._resource import Resource
from databricks.bundles.core._transform import _transform

if TYPE_CHECKING:
    from databricks.bundles.jobs._models.job import Job, JobParam
    from databricks.bundles.pipelines._models.pipeline import Pipeline, PipelineParam

__all__ = ["Resources"]


class Resources:
    """
    Resources is a collection of resources in a bundle.

    Resources class is returned by 'load_resources' function specified in databricks.yml. Each element in
    'python/resources' list is a fully qualified function name that returns an instance of Resources class.
    If there are multiple functions specified in 'python/resources' list, the resources from all functions
    are combined into a single Resources object.

    Example:

    .. code-block:: yaml

        experimental:
          python:
            resources:
              - "resources:load_resources"

    `load_resources` function can be implemented using built-in functions:

    - :meth:`load_resources_from_current_package_module`
    - :meth:`load_resources_from_package_module`
    - :meth:`load_resources_from_modules`
    - :meth:`load_resources_from_module`

    Programmatic construction of resources is supported using :meth:`add_resource` and :meth:`add_job` methods.

    Example:

    .. code-block:: python

        def load_resources(bundle: Bundle) -> Resources:
            resources = Resources()

            for resource_name, config in get_configs():
                job = create_job(config)

                resources.add_job(resource_name, job)

            return resources
    """

    def __init__(self):
        self._jobs = dict[str, "Job"]()
        self._pipelines = dict[str, "Pipeline"]()
        self._locations = dict[tuple[str, ...], Location]()
        self._diagnostics = Diagnostics()

    @property
    def jobs(self) -> dict[str, "Job"]:
        return self._jobs

    @property
    def pipelines(self) -> dict[str, "Pipeline"]:
        return self._pipelines

    @property
    def diagnostics(self) -> Diagnostics:
        """
        Returns diagnostics. If there are any diagnostic errors, bundle validation fails.
        """
        return self._diagnostics

    def add_resource(
        self,
        resource_name: str,
        resource: Resource,
        *,
        location: Optional[Location] = None,
    ) -> None:
        """
        Adds a resource to the collection of resources. Resource name must be unique across all
        resources of the same type.

        :param resource_name: unique identifier for the resource
        :param resource: the resource to add
        :param location: optional location of the resource in the source code
        """

        from databricks.bundles.jobs import Job
        from databricks.bundles.pipelines import Pipeline

        location = location or Location.from_stack_frame(depth=1)

        match resource:
            case Job():
                self.add_job(resource_name, resource, location=location)
            case Pipeline():
                self.add_pipeline(resource_name, resource, location=location)
            case _:
                raise ValueError(f"Unsupported resource type: {type(resource)}")

    def add_job(
        self,
        resource_name: str,
        job: "JobParam",
        *,
        location: Optional[Location] = None,
    ) -> None:
        """
        Adds a job to the collection of resources. Resource name must be unique across all jobs.

        :param resource_name: unique identifier for the job
        :param job: the job to add, can be Job or dict
        :param location: optional location of the job in the source code
        """
        from databricks.bundles.jobs import Job

        job = _transform(Job, job)
        path = ("resources", "jobs", resource_name)
        location = location or Location.from_stack_frame(depth=1)

        if self._jobs.get(resource_name):
            self.add_diagnostic_error(
                msg=f"Duplicate resource name '{resource_name}' for a job. Resource names must be unique.",
                location=location,
                path=path,
            )
        else:
            if location:
                self.add_location(path, location)

            self._jobs[resource_name] = job

    def add_pipeline(
        self,
        resource_name: str,
        pipeline: "PipelineParam",
        *,
        location: Optional[Location] = None,
    ) -> None:
        """
        Adds a pipeline to the collection of resources. Resource name must be unique across all pipelines.

        :param resource_name: unique identifier for the pipeline
        :param pipeline: the pipeline to add, can be Pipeline or dict
        :param location: optional location of the pipeline in the source code
        """
        from databricks.bundles.pipelines import Pipeline

        pipeline = _transform(Pipeline, pipeline)
        path = ("resources", "pipelines", resource_name)
        location = location or Location.from_stack_frame(depth=1)

        if self._pipelines.get(resource_name):
            self.add_diagnostic_error(
                msg=f"Duplicate resource name '{resource_name}' for a pipeline. Resource names must be unique.",
                location=location,
                path=path,
            )
        else:
            if location:
                self.add_location(path, location)

            self._pipelines[resource_name] = pipeline

    def add_location(self, path: tuple[str, ...], location: Location) -> None:
        """
        Associate source code location with a path in the bundle configuration.
        """
        self._locations[path] = location

    def add_diagnostics(self, other: Diagnostics) -> None:
        """
        Add diagnostics from another Diagnostics object.
        :param other:
        :return:
        """
        self._diagnostics = self._diagnostics.extend(other)

    def add_diagnostic_error(
        self,
        msg: str,
        *,
        detail: Optional[str] = None,
        path: Optional[tuple[str, ...]] = None,
        location: Optional[Location] = None,
    ) -> None:
        """
        Report a diagnostic error. If there are any diagnostic errors, bundle validation fails.

        :param msg: short summary of the error
        :param detail: optional detailed description of the error
        :param path: optional path in bundle configuration where the error occurred
        :param location: optional location in the source code where the error occurred
        """
        self.add_diagnostics(
            Diagnostics.create_error(
                msg=msg,
                location=location,
                detail=detail,
                path=path,
            )
        )

    def add_diagnostic_warning(
        self,
        msg: str,
        *,
        detail: Optional[str] = None,
        path: Optional[tuple[str, ...]] = None,
        location: Optional[Location] = None,
    ) -> None:
        """
        Report a diagnostic warning. Warnings are informational and do not cause bundle validation to fail.

        :param msg: short summary of the warning
        :param detail: optional detailed description of the warning
        :param path: optional path in bundle configuration where the warning occurred
        :param location: optional location in the source code where the warning occurred
        """
        self.add_diagnostics(
            Diagnostics.create_warning(
                msg=msg,
                location=location,
                detail=detail,
                path=path,
            )
        )

    def add_resources(self, other: "Resources") -> None:
        """
        Add resources from another Resources object.

        Adds error to diagnostics if there are duplicate resource names.
        """
        for name, job in other.jobs.items():
            self.add_job(name, job)

        for name, pipeline in other.pipelines.items():
            self.add_pipeline(name, pipeline)

        for path, location in other._locations.items():
            self.add_location(path, location)

        self._diagnostics = self._diagnostics.extend(other._diagnostics)



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class DbtCloudTask:
    """
    :meta private: [EXPERIMENTAL]
    """

    connection_resource_name: VariableOrOptional[str] = None
    """
    The resource name of the UC connection that authenticates the dbt Cloud for this task
    """

    dbt_cloud_job_id: VariableOrOptional[int] = None
    """
    Id of the dbt Cloud job to be triggered
    """

    @classmethod
    def from_dict(cls, value: "DbtCloudTaskDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "DbtCloudTaskDict":
        return _transform_to_json_value(self)  # type:ignore


class DbtCloudTaskDict(TypedDict, total=False):
    """"""

    connection_resource_name: VariableOrOptional[str]
    """
    The resource name of the UC connection that authenticates the dbt Cloud for this task
    """

    dbt_cloud_job_id: VariableOrOptional[int]
    """
    Id of the dbt Cloud job to be triggered
    """


DbtCloudTaskParam = DbtCloudTaskDict | DbtCloudTask



import ast
import importlib
import importlib.util
import inspect
import pkgutil
from textwrap import dedent
from types import ModuleType
from typing import Any, Iterable

from databricks.bundles.core._diagnostics import Diagnostics
from databricks.bundles.core._location import Location
from databricks.bundles.core._resource import Resource
from databricks.bundles.core._resources import Resources

__all__ = [
    "load_resources_from_current_package_module",
    "load_resources_from_package_module",
    "load_resources_from_modules",
    "load_resources_from_module",
]

"""
Functions to load resources from modules.

Each resource is a variable that is an instance of `Resource` class. Python variable names are used as resource names.
Resource names for a given resource type must be unique within a bundle.
"""


def load_resources_from_current_package_module() -> Resources:
    """
    Load resources from all submodules of the current package module.
    """

    stack = inspect.stack()
    frame = stack[1].frame
    module = inspect.getmodule(frame)

    if not module:
        raise RuntimeError("Failed to get current module from stack frame")

    return load_resources_from_package_module(module)


def load_resources_from_package_module(
    package_module: ModuleType,
) -> Resources:
    """
    Load resources from all submodules of the given package module.

    :param package_module: package module to load resources from
    """

    package_spec = package_module.__spec__
    if not package_spec:
        raise ValueError(f"Module '{package_module.__name__}' has no __spec__")

    module_names = []
    if package_spec.origin:
        module_names.append(package_spec.name)

    if package_spec.submodule_search_locations:
        package_paths = package_spec.submodule_search_locations

        prefix = package_spec.name + "."

        for loader, module_name, is_pkg in pkgutil.walk_packages(package_paths, prefix):
            module_names.append(module_name)

    module_names.sort()  # create deterministic order
    modules = []
    import_diagnostics = Diagnostics()

    for module_name in module_names:
        try:
            modules.append(importlib.import_module(module_name))
        except Exception as exc:
            import_diagnostics = import_diagnostics.extend(
                Diagnostics.from_exception(
                    exc,
                    summary=f"Error while loading '{module_name}'",
                )
            )

    resources = load_resources_from_modules(modules)
    resources.add_diagnostics(import_diagnostics)

    return resources


def load_resources_from_modules(modules: Iterable[ModuleType]) -> Resources:
    """
    Load resources from the given modules.

    For recursive loading of resources from submodules, use `load_resources_from_package_module`.

    :param modules: list of modules to load resources from
    """

    resources = Resources()

    for module in modules:
        resources.add_resources(load_resources_from_module(module))

    return resources


def load_resources_from_module(module: ModuleType) -> Resources:
    """
    Load resources from the given module.

    For recursive loading of resources from submodules, use `load_resources_from_package_module`.

    :param module: module to load resources from
    """

    resources = Resources()

    try:
        locations = _parse_locations(module)
        fallback_location = Location(file=module.__file__) if module.__file__ else None

        for member_name, member in inspect.getmembers(module, _is_resource):
            location = locations.get(member_name, fallback_location)

            resources.add_resource(member_name, member, location=location)
    except Exception as exc:
        resources.add_diagnostics(
            Diagnostics.from_exception(
                exc,
                summary=f"Error while loading '{module.__name__}'",
            )
        )

    return resources


def _parse_locations(module: ModuleType) -> dict[str, Location]:
    locations = {}
    file = getattr(module, "__file__", "<unknown>")

    try:
        source = inspect.getsource(module)
    except OSError:
        # failed to get source code, non-fatal error because locations are only cosmetic
        return locations

    node = ast.parse(dedent(source), filename=file)

    assert isinstance(node, ast.Module)

    for stmt in node.body:
        if not isinstance(stmt, ast.Assign):
            continue

        for target in stmt.targets:
            if not isinstance(target, ast.Name):
                continue

            var_name = target.id

            locations[var_name] = Location(
                line=stmt.lineno,
                # do conversion: col_offset is 0-based, and column is 1-based
                column=stmt.col_offset + 1,
                file=file,
            )

    return locations


def _is_resource(member: Any) -> bool:
    return isinstance(member, Resource)



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr
from databricks.bundles.jobs._models.condition_task_op import (
    ConditionTaskOp,
    ConditionTaskOpParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class ConditionTask:
    """"""

    left: VariableOr[str]
    """
    The left operand of the condition task. Can be either a string value or a job state or parameter reference.
    """

    op: VariableOr[ConditionTaskOp]
    """
    * `EQUAL_TO`, `NOT_EQUAL` operators perform string comparison of their operands. This means that `“12.0” == “12”` will evaluate to `false`.
    * `GREATER_THAN`, `GREATER_THAN_OR_EQUAL`, `LESS_THAN`, `LESS_THAN_OR_EQUAL` operators perform numeric comparison of their operands. `“12.0” >= “12”` will evaluate to `true`, `“10.0” >= “12”` will evaluate to `false`.
    
    The boolean comparison to task values can be implemented with operators `EQUAL_TO`, `NOT_EQUAL`. If a task value was set to a boolean value, it will be serialized to `“true”` or `“false”` for the comparison.
    """

    right: VariableOr[str]
    """
    The right operand of the condition task. Can be either a string value or a job state or parameter reference.
    """

    @classmethod
    def from_dict(cls, value: "ConditionTaskDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "ConditionTaskDict":
        return _transform_to_json_value(self)  # type:ignore


class ConditionTaskDict(TypedDict, total=False):
    """"""

    left: VariableOr[str]
    """
    The left operand of the condition task. Can be either a string value or a job state or parameter reference.
    """

    op: VariableOr[ConditionTaskOpParam]
    """
    * `EQUAL_TO`, `NOT_EQUAL` operators perform string comparison of their operands. This means that `“12.0” == “12”` will evaluate to `false`.
    * `GREATER_THAN`, `GREATER_THAN_OR_EQUAL`, `LESS_THAN`, `LESS_THAN_OR_EQUAL` operators perform numeric comparison of their operands. `“12.0” >= “12”` will evaluate to `true`, `“10.0” >= “12”` will evaluate to `false`.
    
    The boolean comparison to task values can be implemented with operators `EQUAL_TO`, `NOT_EQUAL`. If a task value was set to a boolean value, it will be serialized to `“true”` or `“false”` for the comparison.
    """

    right: VariableOr[str]
    """
    The right operand of the condition task. Can be either a string value or a job state or parameter reference.
    """


ConditionTaskParam = ConditionTaskDict | ConditionTask



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr, VariableOrDict, VariableOrList

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class PythonWheelTask:
    """"""

    entry_point: VariableOr[str]
    """
    Named entry point to use, if it does not exist in the metadata of the package it executes the function from the package directly using `$packageName.$entryPoint()`
    """

    package_name: VariableOr[str]
    """
    Name of the package to execute
    """

    named_parameters: VariableOrDict[str] = field(default_factory=dict)
    """
    Command-line parameters passed to Python wheel task in the form of `["--name=task", "--data=dbfs:/path/to/data.json"]`. Leave it empty if `parameters` is not null.
    """

    parameters: VariableOrList[str] = field(default_factory=list)
    """
    Command-line parameters passed to Python wheel task. Leave it empty if `named_parameters` is not null.
    """

    @classmethod
    def from_dict(cls, value: "PythonWheelTaskDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "PythonWheelTaskDict":
        return _transform_to_json_value(self)  # type:ignore


class PythonWheelTaskDict(TypedDict, total=False):
    """"""

    entry_point: VariableOr[str]
    """
    Named entry point to use, if it does not exist in the metadata of the package it executes the function from the package directly using `$packageName.$entryPoint()`
    """

    package_name: VariableOr[str]
    """
    Name of the package to execute
    """

    named_parameters: VariableOrDict[str]
    """
    Command-line parameters passed to Python wheel task in the form of `["--name=task", "--data=dbfs:/path/to/data.json"]`. Leave it empty if `parameters` is not null.
    """

    parameters: VariableOrList[str]
    """
    Command-line parameters passed to Python wheel task. Leave it empty if `named_parameters` is not null.
    """


PythonWheelTaskParam = PythonWheelTaskDict | PythonWheelTask



import re
import typing
from dataclasses import Field, fields, is_dataclass
from enum import Enum
from types import NoneType, UnionType
from typing import Any, ForwardRef, Optional, Type, TypeVar, Union, get_args, get_origin

from databricks.bundles.core._variable import (
    Variable,
    VariableOr,
    VariableOrDict,
    VariableOrList,
    VariableOrOptional,
)

__all__ = [
    "_transform",
]

_T = TypeVar("_T")


def _find_union_arg(value: Any, tpe: type) -> Optional[type]:
    """
    Returns the most appropriate type from the list to be used
    in _transform, or None if no type is suitable.
    """

    args = get_args(tpe)

    # If value not None, we support 5 forms:
    #
    # 1. VariableOrOptional[T] = Variable | T | None
    # 2. VariableOr[T]         = Variable | T
    # 3. Optional[T]           = T | None
    # 4. VariableOrList[T]     = VariableOr[list[T]]
    # 5. VariableOrDict[T]     = VariableOr[dict[str, T]]

    for arg in args:
        if _unwrap_variable(arg):
            if _unwrap_variable_path(value):
                return arg
        elif arg is type(None):
            if value is None:
                return type(None)
        else:
            # return the first non-variable and non-None type
            if value is not None:
                return arg

    # allow None to become empty "list" or "dict"
    if value is None:
        for arg in args:
            if _unwrap_list(arg):
                return arg

            if _unwrap_dict(arg):
                return arg

    return None


def _transform_variable_or_list(
    cls: Type[_T],
    value: VariableOrList[Any],
) -> VariableOrList[_T]:
    if isinstance(value, Variable):
        return value

    return [_transform_variable_or(cls, item) for item in value]


def _transform_variable_or_dict(
    cls: Type[_T],
    value: VariableOrDict[Any],
) -> VariableOrDict[_T]:
    if isinstance(value, Variable):
        return value

    return {key: _transform_variable_or(cls, item) for key, item in value.items()}


def _transform_variable_or(
    cls: Type[_T],
    value: VariableOr[Any],
) -> VariableOr[_T]:
    if isinstance(value, Variable):
        return Variable(path=value.path, type=cls)

    return _transform(cls, value)


def _transform_variable_or_optional(
    cls: Type[_T],
    value: VariableOrOptional[Any],
) -> VariableOrOptional[_T]:
    if isinstance(value, Variable):
        return Variable(path=value.path, type=cls)

    if value is None:
        return None

    return _transform(cls, value)


def _transform_field(cls: Type[_T], field: Field, value: Any) -> _T:
    # if we see strings like:
    #
    #   level: "PermissionLevel"
    #
    # we resolve the type for it

    if isinstance(field.type, ForwardRef) or isinstance(field.type, str):
        resolved = typing.get_type_hints(cls)
        return _transform(resolved[field.name], value)

    if origin := get_origin(field.type):
        if origin == Union or origin == UnionType:
            args = get_args(field.type)

            for arg in args:
                if isinstance(arg, (ForwardRef, str)):
                    resolved = typing.get_type_hints(cls)

                    return _transform(resolved[field.name], value)

    if arg := _unwrap_optional(field.type):
        if value is None:
            return None  # type: ignore
        if isinstance(arg, ForwardRef) or isinstance(arg, str):
            resolved = typing.get_type_hints(cls)
            return _transform(resolved[field.name], value)

    return _transform(field.type, value)


def _transform(cls: Type[_T], value: Any) -> _T:
    origin = get_origin(cls)

    if is_dataclass(cls) and isinstance(value, cls):  # type:ignore
        return value

    if isinstance(value, Enum) and isinstance(value, cls):
        return value  # type:ignore

    # if 'cls' is Variable[T], and 'value' is "${xxx}", return Variable(path='xxx', type=T)
    if arg := _unwrap_variable(cls):
        if path := _unwrap_variable_path(value):
            return Variable(path=path, type=arg)  # type:ignore

    if arg := _unwrap_optional(cls):
        if value is None:
            return None  # type:ignore

        return _transform(arg, value)
    elif origin == Union:
        if union_arg := _find_union_arg(value, cls):
            if union_arg is type(None):
                assert value is None

                return None  # type:ignore

            return _transform(union_arg, value)

        raise ValueError(f"Unexpected type: {cls} for {value}")
    elif is_dataclass(cls) and isinstance(value, dict):
        cls_fields = fields(cls)
        field_names = {field.name for field in cls_fields}

        # FIXME: we capture 'cls' in locals(), this should be fixed separately
        if value.get("cls") == cls:
            del value["cls"]

        for key in value:
            if key not in field_names:
                raise ValueError(f"Unexpected field '{key}' for class {cls.__name__}")

        kwargs = {
            field.name: _transform_field(cls, field, value[field.name])
            for field in cls_fields
            if field.name in value
        }

        return cls(**kwargs)  # type:ignore
    elif origin is list:
        if value is None:
            return []  # type:ignore
        [arg] = get_args(cls)

        if isinstance(value, str):
            raise ValueError(f"Unexpected type: {cls} for '{value}'")

        return [_transform(arg, item) for item in value]  # type:ignore
    elif cls is str:
        return str(value)  # type:ignore
    elif cls is int:
        return int(value)  # type:ignore
    elif cls is float:
        return float(value)  # type:ignore
    elif cls is bool:
        if isinstance(value, bool):
            return value  # type:ignore

        if value in ["true", "false"]:
            return value == "true"  # type:ignore

        raise ValueError(f"Unexpected type: {cls} for '{value}'")
    elif issubclass(origin or cls, Enum):
        return cls(str(value))  # type:ignore
    elif cls == NoneType and value is None:
        return value
    elif arg := _unwrap_dict(cls):
        # allow to transform None to empty dict even if type doesn't allow None
        # this is used for "create" method that has different type signature than dataclass
        if value is None:
            return {}  # type:ignore

        [key_arg, value_arg] = arg

        assert key_arg is str

        if not isinstance(value, dict):
            raise ValueError(f"Unexpected type: {cls} for '{value}'")

        out = {key: _transform(value_arg, item) for key, item in value.items()}

        return out  # type:ignore
    else:
        raise ValueError(f"Unexpected type: {cls} for {value}")


def _unwrap_optional(tpe: type) -> Optional[type]:
    if origin := get_origin(tpe):
        if origin == Union or origin == UnionType:
            args = get_args(tpe)

            if len(args) == 2 and args[1] is type(None):
                return args[0]
            elif len(args) == 2 and args[0] is type(None):
                return args[1]

    return None


def _unwrap_list(tpe: type) -> Optional[type]:
    if origin := get_origin(tpe):
        if origin is list:
            args = get_args(tpe)

            if len(args) == 1:
                return args[0]

    return None


def _unwrap_dict(type: type) -> Optional[tuple[type, type]]:
    if origin := get_origin(type):
        if origin is dict:
            args = get_args(type)

            if len(args) == 2:
                return args[0], args[1]

    return None


def _unwrap_variable(tpe: type) -> Optional[type]:
    if origin := get_origin(tpe):
        if origin == Variable:
            return get_args(tpe)[0]

    return None


# Regex for string corresponding to variables.
#
# The source of truth is regex in libs/dyn/dynvar/ref.go
#
# Example:
#   - "${a.b}"
#   - "${a.b.c}"
#   - "${a.b[0].c}"
_base_var_def = r"[a-zA-Z]+([-_]*[a-zA-Z0-9]+)*"
_variable_regex = re.compile(
    r"\$\{(%s(\.%s(\[[0-9]+\])*)*(\[[0-9]+\])*)\}" % (_base_var_def, _base_var_def)
)


def _unwrap_variable_path(value: Any) -> Optional[str]:
    if isinstance(value, str):
        if match := _variable_regex.match(value):
            path = match.group(1)

            if match.start() == 0 and match.end() == len(value):
                return path
            else:
                return None

    # we ignore any generic arguments to Variable, because they
    # only exist for typechecking
    #
    # we should be able to extract path from any variable no
    # matter how it's typed, this makes any variable transformable
    # to any other variable

    if isinstance(value, Variable):
        return value.path

    return None


def _display_type(tpe: type):
    if not get_args(tpe):
        return tpe.__name__

    return repr(tpe)



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr, VariableOrOptional

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class TaskDependency:
    """"""

    task_key: VariableOr[str]
    """
    The name of the task this task depends on.
    """

    outcome: VariableOrOptional[str] = None
    """
    Can only be specified on condition task dependencies. The outcome of the dependent task that must be met for this task to run.
    """

    @classmethod
    def from_dict(cls, value: "TaskDependencyDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "TaskDependencyDict":
        return _transform_to_json_value(self)  # type:ignore


class TaskDependencyDict(TypedDict, total=False):
    """"""

    task_key: VariableOr[str]
    """
    The name of the task this task depends on.
    """

    outcome: VariableOrOptional[str]
    """
    Can only be specified on condition task dependencies. The outcome of the dependent task that must be met for this task to run.
    """


TaskDependencyParam = TaskDependencyDict | TaskDependency



from enum import Enum
from typing import Literal


class JobPermissionLevel(Enum):
    CAN_MANAGE = "CAN_MANAGE"
    CAN_MANAGE_RUN = "CAN_MANAGE_RUN"
    CAN_VIEW = "CAN_VIEW"
    IS_OWNER = "IS_OWNER"


JobPermissionLevelParam = (
    Literal["CAN_MANAGE", "CAN_MANAGE_RUN", "CAN_VIEW", "IS_OWNER"] | JobPermissionLevel
)



from dataclasses import dataclass, field
from typing import Any, TypeVar, Union, get_origin

from databricks.bundles.core._variable import Variable, VariableOr, VariableOrList

__all__ = [
    "Bundle",
]

_T = TypeVar("_T")

_VAR_PREFIX = "var"


@dataclass(frozen=True, kw_only=True)
class Bundle:
    """
    Bundle contains information about a bundle accessible in functions
    loading and mutating resources.
    """

    target: str
    """
    Selected target where the bundle is being loaded. E.g.: 'development', 'staging', or 'production'.
    """

    variables: dict[str, Any] = field(default_factory=dict)
    """
    Values of bundle variables resolved for selected target. Bundle variables are defined in databricks.yml.
    For accessing variables as structured data, use :meth:`resolve_variable`.

    Example:

    .. code-block:: yaml

        variables:
          default_dbr_version:
            description: Default version of Databricks Runtime
            default: "14.3.x-scala2.12"
    """

    def resolve_variable(self, variable: VariableOr[_T]) -> _T:
        """
        Resolve a variable to its value.

        If the value is a variable, it will be resolved and returned.
        Otherwise, the value will be returned as is.
        """
        if not isinstance(variable, Variable):
            return variable

        if not variable.path.startswith(_VAR_PREFIX + "."):
            raise ValueError(
                "You can only get values of variables starting with 'var.*'"
            )
        else:
            variable_name = variable.path[len(_VAR_PREFIX + ".") :]

        if variable_name not in self.variables:
            raise ValueError(
                f"Can't find '{variable_name}' variable. Did you define it in databricks.yml?"
            )

        value = self.variables.get(variable_name)

        # avoid circular import
        from databricks.bundles.core._transform import (
            _display_type,
            _find_union_arg,
            _transform,
            _unwrap_variable_path,
        )

        if nested := _unwrap_variable_path(value):
            can_be_variable = get_origin(variable.type) == Union and _find_union_arg(
                nested, variable.type
            )
            can_be_variable = can_be_variable or get_origin(variable.type) == Variable

            if not can_be_variable:
                display_type = _display_type(variable.type)

                raise ValueError(
                    f"Failed to resolve '{variable_name}' because refers to another "
                    f"variable '{nested}'. Change variable type to "
                    f"Variable[VariableOr[{display_type}]]"
                )

        try:
            return _transform(variable.type, value)
        except Exception as e:
            raise ValueError(f"Failed to read '{variable_name}' variable value") from e

    def resolve_variable_list(self, variable: VariableOrList[_T]) -> list[_T]:
        """
        Resolve a list variable to its value.

        If the value is a variable, or the list item is a variable, it will be resolved and returned.
        Otherwise, the value will be returned as is.
        """

        return [self.resolve_variable(item) for item in self.resolve_variable(variable)]



from enum import Enum
from typing import Literal


class Source(Enum):
    """
    Optional location type of the SQL file. When set to `WORKSPACE`, the SQL file will be retrieved\
    from the local Databricks workspace. When set to `GIT`, the SQL file will be retrieved from a Git repository
    defined in `git_source`. If the value is empty, the task will use `GIT` if `git_source` is defined and `WORKSPACE` otherwise.
    
    * `WORKSPACE`: SQL file is located in Databricks workspace.
    * `GIT`: SQL file is located in cloud Git provider.
    """

    WORKSPACE = "WORKSPACE"
    GIT = "GIT"


SourceParam = Literal["WORKSPACE", "GIT"] | Source



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._resource import Resource
from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import (
    VariableOrDict,
    VariableOrList,
    VariableOrOptional,
)
from databricks.bundles.pipelines._models.event_log_spec import (
    EventLogSpec,
    EventLogSpecParam,
)
from databricks.bundles.pipelines._models.filters import (
    Filters,
    FiltersParam,
)
from databricks.bundles.pipelines._models.ingestion_gateway_pipeline_definition import (
    IngestionGatewayPipelineDefinition,
    IngestionGatewayPipelineDefinitionParam,
)
from databricks.bundles.pipelines._models.ingestion_pipeline_definition import (
    IngestionPipelineDefinition,
    IngestionPipelineDefinitionParam,
)
from databricks.bundles.pipelines._models.notifications import (
    Notifications,
    NotificationsParam,
)
from databricks.bundles.pipelines._models.pipeline_cluster import (
    PipelineCluster,
    PipelineClusterParam,
)
from databricks.bundles.pipelines._models.pipeline_library import (
    PipelineLibrary,
    PipelineLibraryParam,
)
from databricks.bundles.pipelines._models.pipeline_permission import (
    PipelinePermission,
    PipelinePermissionParam,
)
from databricks.bundles.pipelines._models.restart_window import (
    RestartWindow,
    RestartWindowParam,
)
from databricks.bundles.pipelines._models.run_as import RunAs, RunAsParam

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class Pipeline(Resource):
    """"""

    budget_policy_id: VariableOrOptional[str] = None
    """
    :meta private: [EXPERIMENTAL]
    
    Budget policy of this pipeline.
    """

    catalog: VariableOrOptional[str] = None
    """
    A catalog in Unity Catalog to publish data from this pipeline to. If `target` is specified, tables in this pipeline are published to a `target` schema inside `catalog` (for example, `catalog`.`target`.`table`). If `target` is not specified, no data is published to Unity Catalog.
    """

    channel: VariableOrOptional[str] = None
    """
    DLT Release Channel that specifies which version to use.
    """

    clusters: VariableOrList[PipelineCluster] = field(default_factory=list)
    """
    Cluster settings for this pipeline deployment.
    """

    configuration: VariableOrDict[str] = field(default_factory=dict)
    """
    String-String configuration for this pipeline execution.
    """

    continuous: VariableOrOptional[bool] = None
    """
    Whether the pipeline is continuous or triggered. This replaces `trigger`.
    """

    development: VariableOrOptional[bool] = None
    """
    Whether the pipeline is in Development mode. Defaults to false.
    """

    edition: VariableOrOptional[str] = None
    """
    Pipeline product edition.
    """

    event_log: VariableOrOptional[EventLogSpec] = None
    """
    Event log configuration for this pipeline
    """

    filters: VariableOrOptional[Filters] = None
    """
    Filters on which Pipeline packages to include in the deployed graph.
    """

    gateway_definition: VariableOrOptional[IngestionGatewayPipelineDefinition] = None
    """
    :meta private: [EXPERIMENTAL]
    
    The definition of a gateway pipeline to support change data capture.
    """

    id: VariableOrOptional[str] = None
    """
    Unique identifier for this pipeline.
    """

    ingestion_definition: VariableOrOptional[IngestionPipelineDefinition] = None
    """
    The configuration for a managed ingestion pipeline. These settings cannot be used with the 'libraries', 'schema', 'target', or 'catalog' settings.
    """

    libraries: VariableOrList[PipelineLibrary] = field(default_factory=list)
    """
    Libraries or code needed by this deployment.
    """

    name: VariableOrOptional[str] = None
    """
    Friendly identifier for this pipeline.
    """

    notifications: VariableOrList[Notifications] = field(default_factory=list)
    """
    List of notification settings for this pipeline.
    """

    permissions: VariableOrList[PipelinePermission] = field(default_factory=list)

    photon: VariableOrOptional[bool] = None
    """
    Whether Photon is enabled for this pipeline.
    """

    restart_window: VariableOrOptional[RestartWindow] = None
    """
    :meta private: [EXPERIMENTAL]
    
    Restart window of this pipeline.
    """

    root_path: VariableOrOptional[str] = None
    """
    :meta private: [EXPERIMENTAL]
    
    Root path for this pipeline.
    This is used as the root directory when editing the pipeline in the Databricks user interface and it is
    added to sys.path when executing Python sources during pipeline execution.
    """

    run_as: VariableOrOptional[RunAs] = None
    """
    :meta private: [EXPERIMENTAL]
    """

    schema: VariableOrOptional[str] = None
    """
    The default schema (database) where tables are read from or published to.
    """

    serverless: VariableOrOptional[bool] = None
    """
    Whether serverless compute is enabled for this pipeline.
    """

    storage: VariableOrOptional[str] = None
    """
    DBFS root directory for storing checkpoints and tables.
    """

    tags: VariableOrDict[str] = field(default_factory=dict)
    """
    A map of tags associated with the pipeline.
    These are forwarded to the cluster as cluster tags, and are therefore subject to the same limitations.
    A maximum of 25 tags can be added to the pipeline.
    """

    target: VariableOrOptional[str] = None
    """
    Target schema (database) to add tables in this pipeline to. Exactly one of `schema` or `target` must be specified. To publish to Unity Catalog, also specify `catalog`. This legacy field is deprecated for pipeline creation in favor of the `schema` field.
    """

    @classmethod
    def from_dict(cls, value: "PipelineDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "PipelineDict":
        return _transform_to_json_value(self)  # type:ignore


class PipelineDict(TypedDict, total=False):
    """"""

    budget_policy_id: VariableOrOptional[str]
    """
    :meta private: [EXPERIMENTAL]
    
    Budget policy of this pipeline.
    """

    catalog: VariableOrOptional[str]
    """
    A catalog in Unity Catalog to publish data from this pipeline to. If `target` is specified, tables in this pipeline are published to a `target` schema inside `catalog` (for example, `catalog`.`target`.`table`). If `target` is not specified, no data is published to Unity Catalog.
    """

    channel: VariableOrOptional[str]
    """
    DLT Release Channel that specifies which version to use.
    """

    clusters: VariableOrList[PipelineClusterParam]
    """
    Cluster settings for this pipeline deployment.
    """

    configuration: VariableOrDict[str]
    """
    String-String configuration for this pipeline execution.
    """

    continuous: VariableOrOptional[bool]
    """
    Whether the pipeline is continuous or triggered. This replaces `trigger`.
    """

    development: VariableOrOptional[bool]
    """
    Whether the pipeline is in Development mode. Defaults to false.
    """

    edition: VariableOrOptional[str]
    """
    Pipeline product edition.
    """

    event_log: VariableOrOptional[EventLogSpecParam]
    """
    Event log configuration for this pipeline
    """

    filters: VariableOrOptional[FiltersParam]
    """
    Filters on which Pipeline packages to include in the deployed graph.
    """

    gateway_definition: VariableOrOptional[IngestionGatewayPipelineDefinitionParam]
    """
    :meta private: [EXPERIMENTAL]
    
    The definition of a gateway pipeline to support change data capture.
    """

    id: VariableOrOptional[str]
    """
    Unique identifier for this pipeline.
    """

    ingestion_definition: VariableOrOptional[IngestionPipelineDefinitionParam]
    """
    The configuration for a managed ingestion pipeline. These settings cannot be used with the 'libraries', 'schema', 'target', or 'catalog' settings.
    """

    libraries: VariableOrList[PipelineLibraryParam]
    """
    Libraries or code needed by this deployment.
    """

    name: VariableOrOptional[str]
    """
    Friendly identifier for this pipeline.
    """

    notifications: VariableOrList[NotificationsParam]
    """
    List of notification settings for this pipeline.
    """

    permissions: VariableOrList[PipelinePermissionParam]

    photon: VariableOrOptional[bool]
    """
    Whether Photon is enabled for this pipeline.
    """

    restart_window: VariableOrOptional[RestartWindowParam]
    """
    :meta private: [EXPERIMENTAL]
    
    Restart window of this pipeline.
    """

    root_path: VariableOrOptional[str]
    """
    :meta private: [EXPERIMENTAL]
    
    Root path for this pipeline.
    This is used as the root directory when editing the pipeline in the Databricks user interface and it is
    added to sys.path when executing Python sources during pipeline execution.
    """

    run_as: VariableOrOptional[RunAsParam]
    """
    :meta private: [EXPERIMENTAL]
    """

    schema: VariableOrOptional[str]
    """
    The default schema (database) where tables are read from or published to.
    """

    serverless: VariableOrOptional[bool]
    """
    Whether serverless compute is enabled for this pipeline.
    """

    storage: VariableOrOptional[str]
    """
    DBFS root directory for storing checkpoints and tables.
    """

    tags: VariableOrDict[str]
    """
    A map of tags associated with the pipeline.
    These are forwarded to the cluster as cluster tags, and are therefore subject to the same limitations.
    A maximum of 25 tags can be added to the pipeline.
    """

    target: VariableOrOptional[str]
    """
    Target schema (database) to add tables in this pipeline to. Exactly one of `schema` or `target` must be specified. To publish to Unity Catalog, also specify `catalog`. This legacy field is deprecated for pipeline creation in favor of the `schema` field.
    """


PipelineParam = PipelineDict | Pipeline



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class RunAs:
    """
    :meta private: [EXPERIMENTAL]

    Write-only setting, available only in Create/Update calls. Specifies the user or service principal that the pipeline runs as. If not specified, the pipeline runs as the user who created the pipeline.

    Only `user_name` or `service_principal_name` can be specified. If both are specified, an error is thrown.
    """

    service_principal_name: VariableOrOptional[str] = None
    """
    Application ID of an active service principal. Setting this field requires the `servicePrincipal/user` role.
    """

    user_name: VariableOrOptional[str] = None
    """
    The email of an active workspace user. Users can only set this field to their own email.
    """

    @classmethod
    def from_dict(cls, value: "RunAsDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "RunAsDict":
        return _transform_to_json_value(self)  # type:ignore


class RunAsDict(TypedDict, total=False):
    """"""

    service_principal_name: VariableOrOptional[str]
    """
    Application ID of an active service principal. Setting this field requires the `servicePrincipal/user` role.
    """

    user_name: VariableOrOptional[str]
    """
    The email of an active workspace user. Users can only set this field to their own email.
    """


RunAsParam = RunAsDict | RunAs



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrList

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class Filters:
    """"""

    exclude: VariableOrList[str] = field(default_factory=list)
    """
    Paths to exclude.
    """

    include: VariableOrList[str] = field(default_factory=list)
    """
    Paths to include.
    """

    @classmethod
    def from_dict(cls, value: "FiltersDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "FiltersDict":
        return _transform_to_json_value(self)  # type:ignore


class FiltersDict(TypedDict, total=False):
    """"""

    exclude: VariableOrList[str]
    """
    Paths to exclude.
    """

    include: VariableOrList[str]
    """
    Paths to include.
    """


FiltersParam = FiltersDict | Filters
